class Fog < String
   VERSION = '1.0.1'

   # Returns an encrypted +string+ based on the +degree+ that you pass. The
   # default degree is 13. The +degree+ is more or less a salt value.
   #
   def initialize(string, degree=13)
      @degree = degree.to_i
      str = encrypt(string, degree)
      super(str)
   end

   # Decrypts the string using the +degree+ passed to the constructor.
   #
   def decrypt
      unpack('C*').map{ |e| e -= @degree }.pack('C*')
   end

   # The to_s method is identical to inspect, in order to prevent the
   # literal quotation marks from potentially messing up things.
   #
   def to_s
      inspect
   end

   # Decrypts an arbitrary string using +degree+. You must know the degree
   # with which the string was originally encrypted in order for this method
   # to work properly.
   #
   def self.decrypt(string, degree=13)
      string.unpack('C*').map{ |e| e -= degree }.pack('C*')
   end

   private

   # A private method used to encrypt a string based on +degree+.
   #
   def encrypt(str=self, degree=@degree)
      str.unpack('C*').map{ |e| e += degree }.pack('C*')
   end
end
