=begin header

launcher main program

David Maciejak
20030120

=end

RMAGICK_BYPASS_VERSION_TEST = true

if ENV['LANG'] != nil
	ENV['LANG']='en_GB.UTF-8'
	ENV['LANGUAGE']='en_GB.UTF-8'
end

#
# initialize level alert type corresponding to how alert is done according to the severity of the event
#
def init_level_alert_type(h)
	for l in $LEVEL
	  h[l]= [false,false,true,false,false,false,false,false] 
	end
	return 0
end

GNMSLIB = $LOAD_PATH[0]

require 'logger'
require 'thread'

PIXMAP_PATH = "#{GNMSLIB}/../pixmap"
SOUND_DIR = "#{GNMSLIB}/../sound"
SCRIPT_DIR = "#{GNMSLIB}/../script"
TEST_DIR = "#{GNMSLIB}/../test"
PLUGIN_DIR = "#{GNMSLIB}/../plugins"

ROOTMAPADDR="0.0.0.0"
ROOTMAPNAME="ROOT"

NILMAC = "00:00:00:00:00:00"
DEFAULT_CLASS_C_NETMASK = "255.255.255.0"

GNMS_PROJECT_SERVER="gnms.rubyforge.org"
GNMS_PROJECT_URL=""

MAP_SYSLOG_SEV="Inner"

SNMPv1="1"
SNMPv2="2c"

HOSTNODE='h'
NETWORKNODE='n'

#default pixmap size
$pixmap_size = "32"

CRITICAL=5
MAJOR=4
WARNING=3
MINOR=2
NORMAL=1
UNMANAGED=0
UNKNOWN=-1

$status = ["CRITICAL", "MAJOR", "WARNING", "MINOR", "NORMAL", "UNMANAGED", "UNKNOWN"]
$status_color = [0xCC0000FF, 0xFF8800FF, 0xFFDA00FF, 0x69E5E5FF, 0x00CC00FF, 0xBEBEBEFF, 0x9368E5FF]
$status_value = [5, 4, 3, 2, 1, 0, -1]

$LEVEL = ["INFO","WARN","ERR","ALERT","PANIC"]
$level_view="WARN"
$talert_type=["mail", "popup", "beep", "syslog", "sms", "im", "script", "irc"]
$level_alert_type = Hash.new
init_level_alert_type($level_alert_type)

Contact_selected_info=Struct.new("Contact_selected_info", :firstname, :lastname)

$os=Dir["#{PIXMAP_PATH}/os/*"]
$type = Dir["#{PIXMAP_PATH}/type/*"]

$sound_is_played=false

PROJECTNAME = "GNMS"
AUTHORS = [ "David Maciejak", "Dylan Yin" ]
DOCUMENTORS = []

GNMSPPH="just a simple text to badly protect secrets"
GNMSKEY=1978

#creating gnms conf dir
require 'etc'
require 'fileutils'

homedir = Etc.getpwuid.dir
if !homedir
	puts("Error getting home directory ??")
	exit(1)
end

CONF_DIR = "#{homedir}/.#{PROJECTNAME.downcase}/conf"
if !File.exists?(CONF_DIR) or !File.directory?(CONF_DIR)
   File.delete(CONF_DIR) unless !File.exists?(CONF_DIR)
   FileUtils.mkdir_p(CONF_DIR)
end
LOG_DIR = "#{homedir}/.#{PROJECTNAME.downcase}/log"
if !File.exists?(LOG_DIR) or !File.directory?(LOG_DIR)
   File.delete(LOG_DIR) unless !File.exists?(LOG_DIR)
   FileUtils.mkdir_p(LOG_DIR)
end
SNAPSHOT_DIR = "#{homedir}/.#{PROJECTNAME.downcase}/snapshot"
if !File.exists?(SNAPSHOT_DIR) or !File.directory?(SNAPSHOT_DIR)
   File.delete(SNAPSHOT_DIR) unless !File.exists?(SNAPSHOT_DIR)
   FileUtils.mkdir_p(SNAPSHOT_DIR)
end

if $stdout.isatty
  $log = Logger.new(STDOUT)
  $log.level = Logger::INFO
  $log.progname = PROJECTNAME
else
  $log = Logger.new("#{LOG_DIR}/#{PROJECTNAME.downcase}.log", 10, 1024000)
  $log.level = Logger::INFO
  $log.progname = PROJECTNAME
end

begin
	require 'gtk2'
rescue LoadError
	$log.fatal("You must install gtk2 module for ruby")
end

PID_FILE = "#{homedir}/.#{PROJECTNAME.downcase}/#{PROJECTNAME.downcase}.pid"
if File.exists?(PID_FILE)
   is_running = 0
   File.open(PID_FILE, 'r') {|f| pid =  f.read()
   if pid.match(/^\d+$/)
     begin
       Process.kill(0, pid.to_i)
       is_running = pid.to_i
     rescue
       rescue Errno::ESRCH
       is_running = 0
     end
   end
   }
   if is_running > 0
      error_msg = "There is already a running instance of #{PROJECTNAME} pid:#{is_running}"
      $log.fatal(error_msg)
      dialog = Gtk::MessageDialog.new(nil, Gtk::Dialog::DESTROY_WITH_PARENT,
                             Gtk::MessageDialog::ERROR,
                             Gtk::MessageDialog::BUTTONS_OK,
                             error_msg)
      dialog.set_title PROJECTNAME
      dialog.set_icon("#{PIXMAP_PATH}/logo_icon.png")
      dialog.run
      exit(-1)
   else
      $log.info("Old pid file found, maybe #{PROJECTNAME} crashed last time, please report any issues you are encountering")   
   end
end
File.open(PID_FILE, 'w') {|f| f.write($$) }

SQLITE_DB="#{CONF_DIR}/gnms.db"
SQLITE_DB_TEST="#{CONF_DIR}/gnms-test.db"
IDENT_CONF_FILE="#{CONF_DIR}/ident.conf"
GLOBAL_CONF_TEST_FILE="#{CONF_DIR}/gnms-test.conf"
GLOBAL_CONF_FILE="#{CONF_DIR}/gnms.conf"
OIDREFS_CONF_FILE="#{CONF_DIR}/snmp_oidrefs.conf"
SYSLOGREFS_CONF_FILE="#{CONF_DIR}/syslog_refs.conf"
SNMPTRAPREFS_CONF_FILE="#{CONF_DIR}/snmptrap_refs.conf"

#we delete path and extension of the file
for i in [$os,$type]
  ost=[]
  for osi in i
    ostt=osi.split(/\//)
    osttt=(ostt[ostt.size - 1]).split(/\./)
    ost.push osttt[0]
  end
  if ost != nil
    if i==$os
     $os=ost.sort
    else
     $type=ost.sort
    end
  end
end

#
#	check ruby and gtk version
#
def verify_system_conf()
	ruby_cmd_version = RUBY_VERSION 
        $log.info("Using Ruby version #{RUBY_VERSION}")
	ruby_version="^1\.[89]"
	if ruby_cmd_version.match(ruby_version)
		if (defined? Gtk) and !Gtk.check_version(2,4,0)
			return 0
		else
			$log.error("This tool needs at least GTK version 2.4")
		end
	else
		$log.error("This tool needs at least Ruby 1.8")
	end
	return 1
end


#
#	return logger module level according to event log severity
#
def txtlvl_to_loggerlvl(str)
	case str
	when "Debug"
		return Logger::DEBUG
	when "Info"
		return Logger::INFO
	when "Warning"
		return Logger::WARN
	when "Error"
		return Logger::ERROR
	else
		return Logger::INFO
	end
end

####################################################################################
#
# let's rock!
#

require "#{GNMSLIB}"+'/version'
require "#{GNMSLIB}"+'/main'

#verify ruby/gtk/gnome conf
unless verify_system_conf == 0
	$log.error("System configuration is not ok")
	exit(1)
else
	$log.info("System configuration is ok")

$log.info("Running GNMS version #{GNMSVERSION}")

Gtk::RC.parse_string <<EOS

style "tooltips"

{
  bg[NORMAL] = {0.94, 0.90, 0.54}
}

widget "gtk-tooltips" style "tooltips"
EOS



module Gtk
    # Thread-safety stuff.
    # Loosely based on booh, by Guillaume Cottenceau.

    PENDING_CALLS_MUTEX = Mutex.new
    PENDING_CALLS = []

    def self.thread_protect(&proc)
        if Thread.current == Thread.main
            proc.call
        else
            PENDING_CALLS_MUTEX.synchronize do
                PENDING_CALLS << proc
            end
        end
    end

    def self.thread_flush
        if PENDING_CALLS_MUTEX.try_lock
            for closure in PENDING_CALLS
                closure.call
            end
            PENDING_CALLS.clear
            PENDING_CALLS_MUTEX.unlock
        end
    end

    def self.init_thread_protect
        Gtk.timeout_add(100) do
            PENDING_CALLS_MUTEX.synchronize do
                for closure in PENDING_CALLS
                    closure.call
                end
                PENDING_CALLS.clear
            end
            true
        end
    end
end

Gtk.init_thread_protect 

  begin
	Gtk.init
  rescue
	$log.fatal("Can't init GTK, is your display correct ?")	
	exit(1)
  end

  #nagscreen

  #window = Gtk::Window.new
  #window.set_default_size(300,150)
  #window.set_window_position(Gtk::Window::POS_CENTER)
  #window.set_title "GNMS"
  #window.set_decorated(false)
  #window.set_modal(true)
  #hbox = Gtk::VBox.new
  #hbox.add(Gtk::Image.new("#{PIXMAP_PATH}/logo.jpg"))
  #lb=Gtk::Label::new("Starting")
  #lb.show
  #hbox.add lb
  #hbox.show_all 
  #window.add hbox	
  #window.show_all

  #Thread.start {
  launch_gnms_main()
	#}

  Gtk.main

end
