require "#{GNMSLIB}"+'/node_property'
require "#{GNMSLIB}"+'/node'
require "#{GNMSLIB}"+'/cmd_parse'
require "#{GNMSLIB}"+'/node_listener'
require "#{GNMSLIB}"+'/gui/adddelnode'
require "#{GNMSLIB}"+'/gui/base_window'
require "#{GNMSLIB}"+'/gui/widgets.rb'
require "#{GNMSLIB}"+'/notify/alarm'

class EventWindow

EVENT_WINDOW_TITLE="Event Browser"

#event type
CORE_EVENT_TYPE="core"
PORT_EVENT_TYPE="port"
AVAIL_EVENT_TYPE="interface"
TRAP_EVENT_TYPE="trap"
CUSTOM_EVENT_TYPE="custom"
SNMP_EVENT_TYPE="snmp"
WMI_EVENT_TYPE="wmi"
JMX_EVENT_TYPE="jmx"
SYSLOG_EVENT_TYPE="syslog"
HTTP_EVENT_TYPE="http"

#end event type
#
#	return if window is visible
#
def visible?()
	return @window.visible?()
end

#
#	show the window
#
def show()
	if $canvas.fullscreen?()
	  @window.set_modal(true)
	  @window.set_transient_for($win)
	end
	@window.deiconify()
	return @window.show()
end

#
#	hide the window
#
def hide()
	if $canvas.fullscreen?()
	  @window.set_modal(false)
	  @window.set_transient_for(nil)
	end
	return @window.hide()
end

def filter_by_sev(sev)
	$level_view = sev
	if @filter_node
		filter_by_node(@filter_node)
	else
		filter_by_node()
	end
end

#
#	show logs for this specific node, node=nil to show all logs
#
def filter_by_node(node=nil)
	@list.clear()
	if node
	  @filter_active=true
	  @filter_node=node
	  @event_strings.each {|pos, category, src_module, time, ip, summary, state|
		if state != "A"
		  if filter_on_node(ip) && ($LEVEL.index(category)>=$LEVEL.index($level_view))
			 text = []
   		 text.push pos
   		 text.push category
   		 text.push src_module
   		 text.push time
   		 text.push ip
   		 text.push summary
			 @list.prepend(text)
		  end
		end
	  }
	  @window.set_title(EVENT_WINDOW_TITLE + " filter #{$level_view} from #{node.ip}")
	else
	  @filter_active=false
	  @filter_node=nil
	  @event_strings.each {|pos, category, src_module, time, ip, summary, state|
		if state != "A"
		  if ($LEVEL.index(category)>=$LEVEL.index($level_view))
			  text = []
   		  text.push pos
   		  text.push category
   		  text.push src_module
   		  text.push time
   		  text.push ip
   		  text.push summary
			  @list.prepend(text)
		  end
		end
	  }
	  @window.set_title(EVENT_WINDOW_TITLE)
	end
end

#
#	define if ip is part of the filter tag
#
def filter_on_node(ip_str)
	if @filter_node == nil
		return false
	end
	if exist_network(@filter_node.ip)
		#we want to retrieve events from all a network
		return $network[@filter_node.ip].contain_host(ip_str)		
	end
	return ip_str == @filter_node.ip
end

#
# add event in list
# state can be normal N or acknowledge A
#
def add_event(src_module, category, node, summary, datetime=nil, state='N')
  if node
    ip_str = node.ip
    #puts "in add_event #{ip_str} #{summary}"
    text = []
    index_pos=@event_strings.length()+1
    text.push index_pos.to_s
    text.push category
    text.push src_module
    if datetime == nil
      datetime=Time.new.to_s
    end
    text.push datetime
    text.push ip_str
    text.push summary

    if filter_on_node(ip_str) || @filter_active == false
      if $LEVEL.index(category)>=$LEVEL.index($level_view)
        @buff.put Marshal.load(Marshal.dump(text))
      end
    end
    text.push state
    #in event_strings there is one more column: state
    @event_strings.push(text)

    alarm_handle(src_module, category, ip_str, summary, datetime)
    update_node_notification(get_status_from_level(category), ip_str)
    update_statusbar()
  end
end

def hide_event()
	iter = @list.selection.selected
	if iter != nil
	  #set state to aknowledge
  	  list_id=@list.get_value(iter,1)
	  event_id=list_id.to_i()-1
     if @event_strings[event_id] != nil
		@event_strings[event_id][6]='A'
		#real time events sev need to be caught
		if @event_strings[event_id][2] != CORE_EVENT_TYPE
		  event_type=@event_strings[event_id][2]
		  max_lev=nil
		  @event_strings.each {|pos, category, src_module, time, ip, summary, state|
			if (src_module == event_type) && (state != "A")
				if (max_lev != nil)
					if ($LEVEL.index(category) > $LEVEL.index(max_lev))
						max_lev=category
					end
				else
					max_lev=category
				end
			end
		  }
			if max_lev != nil
				max_lev_status=get_status_from_level(max_lev)
				if $host[@event_strings[event_id][4]] != nil
					$host[@event_strings[event_id][4]].set_severity(event_type, $status.index(max_lev_status))
				end
			else
				if $host[@event_strings[event_id][4]] != nil
					$host[@event_strings[event_id][4]].set_severity(event_type, $status.index("UNMANAGED"))
				end				
			end
		end
		#update notification icon
		print "in hide_event #{@event_strings[event_id][4]} #{$status_value[$status.index(get_status_from_level(@event_strings[event_id][1]))]}\n"
		ip = @event_strings[event_id][4]
		if $host[ip] and $host[ip].node_view
		   sev=$status_value[$status.index(get_status_from_level(@event_strings[event_id][1]))]
		   if sev > NORMAL
		      $host[ip].node_view.del_notification(sev)
		   end
		end
		#remove from gui
		@list.remove(iter) unless iter == nil
	  end
	end
end

def update_node_notification(sev, ip)
   sev_value = $status_value[$status.index(sev)]
   if sev_value > NORMAL
   	$host[ip].node_view.add_notification(sev_value)
   end
end

def acknowledge_event()
	hide_event()
end

def stop_analyzer()
	 if @event_analyzer_thread.alive?
		 @event_analyzer_thread_status = 0
		 Thread.kill(@event_analyzer_thread)
	 end
end


def analyze_event()
   while @buff.has_more?     
      event=@buff.get
      #puts "in analyze_event: #{@buff.has_more?} #{event}"
      @list.prepend(event)      
   end
end

def clean()
  @event_strings=[]
  @filter_active=false
  @filter_node=nil
end

def initialize()

  @statusbar=nil
  @in_statusbar = Mutex.new
  @list=nil
  clean()
  
  #################################################
  #create thread to handle new events
  #
  @buff=MsgBuffer.new(1)
  @event_analyzer_thread_status = 1
  @event_analyzer_thread = Thread.start do
        while (@event_analyzer_thread_status)
      	   analyze_event if @buff.full?
      	   sleep(1)
	   Thread.pass
   	end
        $log.debug("end of EventAnalyzer thread")
    end

  #################################################
  #GUI part
  #
  @window=Gtk::Window.new
  @window.signal_connect("delete_event") do
		self.hide()
  end

  @window.signal_connect("key_press_event") {|w,e|
    if e.keyval == Gdk::Keyval::GDK_Escape
		self.hide()
    end
  }

  @window.set_size_request(600, 100)
  @window.set_title(EVENT_WINDOW_TITLE)
  @window.set_icon("#{PIXMAP_PATH}/logo_icon.png")

  box1 = Gtk::VBox.new(false, 0)
  @window.add(box1)

  box1.show
  box11 = Gtk::VBox.new(false, 0)
  box11.show

#------------begin scrolledbar--------------------------------------
  scrolled_window = Gtk::ScrolledWindow.new(nil, nil)
  scrolled_window.border_width=0
  scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC,
                             Gtk::POLICY_AUTOMATIC)
  # scrolled_window.vscrollbar.unset_flag(Gtk::CAN_FOCUS)
  box1.pack_start(scrolled_window, true, true, 0)
  
  scrolled_window.add_with_viewport(box11)
  box11.set_focus_vadjustment(scrolled_window.vadjustment)
  scrolled_window.show
#----------------end scrolledbar-----------------------------------


@list = Gtk::CList::new(["Event ID", "Level", "Module","Timestamp", "Node", "Summary" ])
@list.set_sort_column(1,4)
#@list.selection.set_mode(Gtk::SELECTION_MULTIPLE)
# Create a popup menu
menu = Gtk::Menu.new

item_ack = Gtk::MenuItem.new("Acknowledge event")
item_ack.signal_connect("activate") { acknowledge_event() }
menu.append(item_ack)

item_clear = Gtk::MenuItem.new("Clear display")
item_clear.signal_connect("activate") { @list.clear() }
menu.append(item_clear)

item_remove_filter = Gtk::MenuItem.new("Remove src filter")
item_remove_filter.signal_connect("activate") { filter_by_node() }
menu.append(item_remove_filter)


map_menu_severity=Gtk::MenuItem.new("Severity threshold")
map_menu_severity_list=Gtk::Menu.new
radio_menu_grp=nil
for level in $LEVEL
	rmi=Gtk::RadioMenuItem.new(radio_menu_grp,"#{level}")
	if level == $level_view
		rmi.set_active(true)
	end
   rmi.signal_connect("activate") do |widget|
			if widget.active?
		       filter_by_sev(widget.child.label)
			end
    end
	radio_menu_grp=rmi.group
	map_menu_severity_list.append(rmi)
end
map_menu_severity.set_submenu(map_menu_severity_list)
menu.append(map_menu_severity)

menu.show_all

@list.signal_connect("cursor-changed") do |widget, event|
	iter=widget.selection.selected
	ip=widget.get_value(iter,5)

	if ip != nil
		  if $network.has_key?(ip)
			  set_map($network[ip].map)
			  $network[ip].node_view.select()
		  else
			  #verify that node exist
			  if $host[ip] != nil
			  	 set_map($host[ip].map)
				 $host[ip].node_view.select()
			  end
		  end
	end
end

@list.signal_connect("button_press_event") do |widget, event|
  if event.kind_of? Gdk::EventButton  and event.button == 3
		if @list.nb_event > 0
			item_clear.set_sensitive(true)
		else
			item_clear.set_sensitive(false)
		end
		if @filter_active
			item_remove_filter.set_sensitive(true)
		else
			item_remove_filter.set_sensitive(false)
		end
	    menu.popup(nil, nil, event.button, event.time)

  end
end

@list.signal_connect("key_press_event") {|w, e| 
  if (e.keyval == Gdk::Keyval::GDK_Delete) || (e.keyval == Gdk::Keyval::GDK_d)
    hide_event()
  end
}

 @list.show
 box11.add @list

#---------------------begin statusbar---------

    @statusbar = Gtk::Statusbar.new
    
    vvbox = Gtk::VButtonBox.new
    vvbox.border_width=0
    vvbox.set_layout_style(Gtk::ButtonBox::END)
    vvbox.set_spacing(0)
    vvbox.set_size_request(200, 20)
    vvbox.show

    box1.pack_start(@statusbar,false,false,0)
    
    @statusbar.show
    @statusbar.signal_connect("text_popped") do |o, mid, text|
      popped(mid,text)
    end
    @statusbar.signal_connect("text_pushed") do |o, mid, text|
      pushed(mid,text)
    end

#---------------------end status bar----------

end

#
# update status bar (active host) / (total host)
# also update the statusicon if exists
#
def update_statusbar()
  @in_statusbar.synchronize {
    hup=0
    hdown=0
    if @statusbar != nil
      begin
	$host.each_key {|ip|
          if $host[ip] && ($host[ip].status == NORMAL)
            hup=hup+1
          else
            hdown=hdown+1
          end
	}
      rescue Exception => msg
	$log.error(msg)
      end
      str="Active Host" + ($host.size<2?" ":"s ") + "#{hup}/#{$host.size}"
      @statusbar.push(1, str)
      if $icon && $icon.embedded?
	$icon.set_tooltip(str)
      end
    end
  }
end

def updating_statusbar?()
   @in_statusbar.locked?
end

 def save_event()
	iter_zero=@list.model.get_iter("0")
	if iter_zero != nil
	  #we extract the timestamp of the first entry
	  bbeg= @list.get_value(iter_zero,4)
	  vall=@list.nb_event-1
	  #we extract the timestamp of the last entry
	  bend=@list.get_value(@list.model.get_iter("#{vall}"),4)
	
	  #filename is base on the timestamp of the first and last entry
	  begin
		  filepath="#{LOG_DIR}/#{bend}-#{bbeg}"
		  $log.info("Saving log to #{filepath}")
		  fic = File.new(filepath,'w')
		  for row in 0..@list.nb_event-1
	   	  	fic.puts(@list.get_value(@list.model.get_iter("#{row}"),4)+"\t"+@list.get_value(@list.model.get_iter("#{row}"),3)+"\t"+@list.get_value(@list.model.get_iter("#{row}"),2)+"\t"+@list.get_value(@list.model.get_iter("#{row}"),5)+"\t"+@list.get_value(@list.model.get_iter("#{row}"),6))
		  end
		  fic.close
	 rescue
	 	$log.error("Can't write log to #{LOG_DIR}")
	 end
	end
end

def pushed(mid,text)
end

def popped(mid,text)
end

end
