module IPCalc

def IPCalc.bits_to_netmask(bits)
	bits=bits.to_i()
	if bits <=0
		return "0.0.0.0"
	end
	if bits >= 32
		return "255.255.255.255"
	end

	cpt=0
	cpt_byte=8
	result=""
	result_val=0
	while bits >= 0 ||cpt <= 32
		val=1
		val=0 unless bits > 0
		if cpt%8==0
			if cpt != 0
			result+="#{result_val}"
			result+="." unless cpt == 32
			cpt_byte=8
			result_val=0
			end
		end
		result_val+=2**(cpt_byte-1)*val
		cpt_byte-=1
		cpt+=1
		bits-=1
	end
	return result
end

def IPCalc.netmask_to_bits(netmask)
	#verify if netmask is an 4 dot format
	if netmask.nil? or netmask.empty? or !isValidIPv4(netmask)
		return nil
	end
	netmask_t=netmask.split(/\./)
	result_val=0

	for val in netmask_t
   	cpt_byte=8
		val_tmp=val.to_i()
		while val_tmp > 0
			val_tmp-=2**(cpt_byte-1)
			cpt_byte-=1
			result_val+=1
		end
	end
	return result_val
end


#
#	take an integer
#
def IPCalc.get_wildcard(mask)
	mask_local=mask
	if mask.to_s().match('^\d+$')
		mask_local=bits_to_netmask(mask)
	end
	#verify if mask_local is an 4 dot format
	if !isValidIPv4(mask_local)
		return nil
	end
	netmask_t=mask_local.split(/\./)
	result=""
	for val in netmask_t
		val_tmp=val.to_i()
		result+="#{255-val_tmp}."
	end
	return result.chop
end

def IPCalc.get_network_and_mask(addr)
	#verify if addr is an 4 dot format string
	if !isValidIPv4(addr)
		return nil
	end
	result_t=[]
	mask=class_type_default_mask(addr)
	wildcard=get_wildcard(mask)
	wildcard_t=wildcard.split(/\./)
	addr_t=addr.split(/\./)
	cpt=0
	for wildc in wildcard_t
		if wildc != "0"
			addr_t[cpt]="0"
		end
		cpt+=1
	end
	result=addr_t.join(".")
	if result == addr
		return nil
	else
		result_t.push result
		result_t.push mask
		return result_t
	end
end

def IPCalc.get_network_addr(addr, mask)
	#verify if addr is an 4 dot format string
	if !isValidIPv4(addr)
		return nil
	end
	wildcard=get_wildcard(mask)
	wildcard_t=wildcard.split(/\./)
	addr_t=addr.split(/\./)
	cpt=0
	for wildc in wildcard_t
		if wildc != "0"
			addr_t[cpt]="0"
		end
		cpt+=1
	end
	return addr_t.join(".")
end

def IPCalc.get_broadcast(network, mask)
	#verify if network is an 4 dot format string
	if !isValidIPv4(network)
		return nil
	end
	wildcard=get_wildcard(mask)
	wildcard_t=wildcard.split(/\./)
	addr_t=network.split(/\./)
	cpt=0
	for wildc in wildcard_t
		if wildc=="0"
			wildcard_t[cpt]=addr_t[cpt] unless addr_t[cpt] == "0"
		end
		cpt+=1
	end
	return wildcard_t.join(".")

end

def IPCalc.nb_host(netmask)
	mask=netmask
	if !netmask.to_s().match('^\d+$')
		mask=netmask_to_bits(netmask)
	end
	if mask==nil
		return nil
	end
	mask=mask.to_i()
	result=2**(32-mask)-2
	result=1 if result<=0
	return result
end

def IPCalc.to_bitwise(four_dot_addr)
	#verify if four_dot_addr is an 4 dot format string
	if !isValidIPv4(four_dot_addr)
		return nil
	end
	four_dot_addr_t=four_dot_addr.split(/\./)
	result=""
	for four_dot_split in four_dot_addr_t
		byte_val=8
		while four_dot_split.to_i>0
			if four_dot_split.to_i >= 2**(byte_val-1)
				result+="1"
			four_dot_split=four_dot_split.to_i()-2**(byte_val-1)
			else
				result+="0"
			end
			byte_val-=1
		end
		result+="0"*byte_val
		result+="."
	end
	return result.chop
end

def IPCalc.class_type_name(network)
	class_name="class"
	mask=network
	if network.to_s().match('^\d+$')
		mask=bits_to_netmask(network)
	end
	if mask==nil
		return nil
	end
	bitmask=to_bitwise(mask)
	if bitmask.to_s().match('^0')
		return class_name+" "+"A"
	elsif bitmask.to_s().match('^10')
		return class_name+" "+"B"
	elsif bitmask.to_s().match('^110')
		return class_name+" "+"C"
	elsif bitmask.to_s().match('^1110')
		return class_name+" "+"D"
	elsif bitmask.to_s().match('^1111')
		return class_name+" "+"E"
	end
end

def IPCalc.class_type_default_mask(addr)
	#verify if addr is an 4 dot format string
	if !isValidIPv4(addr)
		return nil
	end
	class_name="class"
	mask=addr
	if addr.to_s().match('^\d+$')
		mask=bits_to_netmask(netmask)
	end

	bitmask=to_bitwise(mask)
	if bitmask.to_s().match('^0')
		return 8
	elsif bitmask.to_s().match('^10')
		return 16
	elsif bitmask.to_s().match('^110')
		return 24
	elsif bitmask.to_s().match('^1110')
		return 4
	elsif bitmask.to_s().match('^1111')
		return 4
	end
end

#
# launch ipcalculator ip can be a 4 dot addr or addr/bitmask
#
def IPCalc.ipcalc_detail(addr_str)
	addr=""
	mask=""
	if isValidIPv4(addr_str)
		#we need default mask
		addr=addr_str
		mask=class_type_default_mask(addr_str)
	elsif addr_str.match('^[\d.]+\/\d+$')
		addr,mask=addr_str.split(/\//)
	end
	result=
	"Address: "+addr+"\n"+
	"NetMask: "+mask+"\n"+
	"Wildcard: "+get_wildcard(mask)+"\n"+
	"Network: "+get_network_addr(addr,mask)+"\n"+
	"Broadcast: "+get_broadcast(addr,mask)+"\n"+
	"Hosts: "+nb_host(mask).to_s()+"\n"+
	"Type: "+class_type_name(addr)+"\n\n";

	#HostMin:
	#HostMax:
  return result
end

#puts bits_to_netmask(ARGV[0].to_i)
#puts netmask_to_bits("0.0.0.0")
#puts get_wildcard("255.255.252.0")
#puts get_wildcard(22)
#puts get_broadcast("12.168.1.0",22)
#puts get_network_addr("12.168.1.2",22)
#puts nb_host(22)
#puts to_bitwise("192.168.1.1")
#puts class_type_name("18.0.0.0")
#puts get_network_and_mask("10.10.1.1")

#to do : hostmin hostmax priv/pub
end
