#
# module to use snmp linux command
#
require 'timeout'
require "#{GNMSLIB}"+'/config_global'

#
#	return the value of the entry
#
def get_snmp_OID(node, oid)

  #get values from node network
  version=node.get_default_inherit_snmp_version()
  port=node.get_default_inherit_snmp_port()
  timeout=node.get_default_inherit_snmp_timeout()
  retr=node.get_default_inherit_snmp_retry()

  community_read = $config.read_community
  community_write = $config.write_community
  if node.get_account_id("snmp")
    community_read, community_write = node.get_account_id("snmp")
  end

  begin
    #print "get_snmp_OID: ", node.ip, " ", community_read, " ", community_write, " ", get_snmp_version(version), " ", port.to_i, " ", timeout.to_i, " ", retr.to_i,"\n"
    SNMP::Manager.open(:Host => node.ip, :Community => community_read, :WriteCommunity => community_write, :Version => get_snmp_version(version), :Port => port.to_i, :Timeout => timeout.to_i, :Retries => retr.to_i) do |manager|
      response = manager.get(oid)
      response.each_varbind do |vb|
        return vb.value.to_s
      end
    end
  rescue SNMP::RequestTimeout    
    $log.debug("Error in get_snmp_OID: #{node.ip} not responding")
    return "Error"  
  rescue Exception => msg
    $log.error("Error in get_snmp_OID: #{msg}")
    return "Error"
  end
end

#
#	return an array of the value of the entry
#
def walk_snmp_OID(node, oid)

  #get values from node network
  version=node.get_default_inherit_snmp_version()
  port=node.get_default_inherit_snmp_port()
  timeout=node.get_default_inherit_snmp_timeout()
  retr=node.get_default_inherit_snmp_retry()

  community_read = $config.read_community
  community_write = $config.write_community
  if node.get_account_id("snmp")
    community_read, community_write = node.get_account_id("snmp")
  end

  begin
    snmp_manager=SNMP::Manager.open(:Host => node.ip, :Community => community_read, :WriteCommunity => community_write, :Version => get_snmp_version(version), :Port => port.to_i, :Timeout => timeout.to_i, :Retries => retr.to_i) do |manager|
      resultt=[]
      manager.walk(oid) {|vb|
        resultt.push(vb.to_s)
      }
      return resultt
    end
  rescue SNMP::RequestTimeout
    $log.debug("Error in get_snmp_OID: #{node.ip} not responding")
    return "Error"
  rescue Exception => msg
    $log.error("Error in get_snmp_OID: #{msg}")
    return "Error"
  end
end

#
#	set the value of the entry
#
def set_snmp(node, oid, value)

  #get values from node network
  version=node.get_default_inherit_snmp_version()
  port=node.get_default_inherit_snmp_port()
  timeout=node.get_default_inherit_snmp_timeout()
  retr=node.get_default_inherit_snmp_retry()

  community_read = $config.read_community
  community_write = $config.write_community
  if node.get_account_id("snmp")
    community_read, community_write = node.get_account_id("snmp")
  end

  begin
    if oid ==nil || value == nil
      $log.error("Error in set_snmp_OID: oid/value is null")
      return "Error"
    end
    SNMP::Manager.open(:Host => node.ip, :Community => community_read, :WriteCommunity => community_write, :Version => get_snmp_version(version), :Port => port.to_i, :Timeout => timeout.to_i, :Retries => retr.to_i) do |manager|
      varbind=SNMP::VarBind.new(oid,SNMP::OctetString.new(value))
      response = manager.set(varbind)
      if response.error_status().to_s != "noError"
        return response.error_status().to_s
      end
    end
    return 0
  rescue Exception => msg
    $log.error("Error in set_snmp_OID: #{msg}")
    return "Error"
  end
end

#
# Return uptime of an host
#
def get_snmp_uptime(node)
  if node.status == CRITICAL
    return "Error"
  end
  return get_snmp_OID(node, "sysUpTime.0")
end

#
# Return location of an host
#
def get_snmp_location (node)
  if node.status == CRITICAL
    return "Error"
  end
  return get_snmp_OID(node, "sysLocation.0")
end

#
# Return contact of an host
#
def get_snmp_contact(node)
  if node.status == CRITICAL
    return "Error"
  end
  return get_snmp_OID(node, "sysContact.0")
end

#
# Return the description of an host
#
def get_snmp_description(node)
  if node.status == CRITICAL
    return "Error"
  end
  return get_snmp_OID(node, "sysDescr.0")
end

#
# Return the name of a node
#
def get_snmp_name(host)
  if node.status == CRITICAL
    return "Error"
  end
  return get_snmp_OID(node, "sysName.0")
end

#
# Return the MAC address of the ip give in argument,
# use snmp
#
def mac(ip)
  #first try to solve it by the help of local arp table
  temp=mac_tablelocal(ip)

  if (temp!="")
    return temp.chomp
  end
  if !$config.snmp_mon || $host[ip].status == CRITICAL
      return
  end
  val=get_snmp_OID($host[ip], "ipAdEntIfIndex.#{ip}")
	if !val || val == "Error"
    return
  end
  result=get_snmp_OID($host[ip], "ifPhysAddress.#{val}")
  #expecting the MAC address as a string
  if !result.ascii_only?
    #if binary, we tried to convert it
    result = result.unpack("H2H2H2H2H2H2").join(":")
  end

  if !isValidMac(result)
    return
  end
  return result
end
