require 'resolv'

class SyslogAnalyzer

  def initialize(buff)
    @buff=buff

    @sender = Thread.start do
   	while $config.syslog_mon
      	   send_messages if buff.full?
      	   sleep(1)
           Thread.pass
   	 end
	$log.debug("end of SyslogAnalyzer thread")
    end
  end
  
  def get_thread
    @sender
  end

		#extract: DATETIME|HOST<PRI>MSG
  def format_syslog_packet(msg)
	  msg.scan(/^([^|]+)\|([\d.]+)<(\d+)>(.+)$/) do |date,src,pri,msg|
		puts "dans format_syslog_packet"
		  return pri, date, src, msg
	  end
  end

#
#	need syslog sev and return a gnms sev (an index of $status)
#
	def map_syslog_sev(sev)
	  case sev
	  when "Emergency"
		  return 0
	  when "Alert"
		  return 0
	  when "Critical"
		  return 1
	  when "Error"
		  return 1
	  when "Warning"
		  return 2
	  when "Notice"
		  return 3
	  when "Informational"
		  return 4
	  when "Debug"
		  return 4
	  end
	end

  def extract_pri(pri_val)

	#info taken from RFC 3164

	severity=Array.new(8,nil)
	severity[0]="Emergency"
	severity[1]="Alert"
	severity[2]="Critical"
	severity[3]="Error"
	severity[4]="Warning"
	severity[5]="Notice"
	severity[6]="Informational"
	severity[7]="Debug"

	facility=Array.new(24,nil)
	facility[0]="kernel messages"
	facility[1]="user-level messages"
   facility[2]="mail system"
   facility[3]="system daemons"
   facility[4]="security/authorization messages (note 1)"
   facility[5]="messages generated internally by syslogd"
   facility[6]="line printer subsystem"
   facility[7]="network news subsystem"
   facility[8]="UUCP subsystem"
	facility[9]="clock daemon (note 2)"
	facility[10]="security/authorization messages (note 1)"
	facility[11]="FTP daemon"
	facility[12]="NTP subsystem"
	facility[13]="log audit (note 1)"
	facility[14]="log alert (note 1)"
	facility[15]="clock daemon (note 2)"
   facility[16]="local use 0  (local0)"
   facility[17]="local use 1  (local1)"
   facility[18]="local use 2  (local2)"
   facility[19]="local use 3  (local3)"
   facility[20]="local use 4  (local4)"
   facility[21]="local use 5  (local5)"
   facility[22]="local use 6  (local6)"
   facility[23]="local use 7  (local7)"

	for i in 0..severity.size()
		val=(pri_val.to_i()-i) % 8
		if val==0
			fsev=i
			ffacility=(pri_val.to_i()-i) / 8
			break
		end
	end
	#print "facility: ",facility[ffacility], " severity: ", severity[fsev],"\n"
	return Array[facility[ffacility], severity[fsev]]
 end

  private
  def send_messages
    while @buff.has_more?
      syslog_msg=@buff.get
		if syslog_msg != nil
			print "dans syslog analyzer ",syslog_msg
			#extract pri, date, src node, msg
			pri, date, src, msg=format_syslog_packet(syslog_msg)
			if msg != nil
				if $host[src] != nil && $host[src].syslog_monitoring.size() > 0
					$host[src].syslog_monitoring.each_value do |syslog_mon|
						if syslog_mon.active?()
      				  msg.scan($syslog_refs[syslog_mon.get_syslog_ref()].get_regex()) do 
						  $log.debug "syslog ref: #{syslog_mon.get_syslog_ref()}"
							  #year,month,day,hour,min,sec=parsedate(date)
							  #test if we want default sev of syslog msg
							  if syslog_mon.raw_severity()>0
								  $event_win.add_event(EventWindow::SYSLOG_EVENT_TYPE, get_level_from_status($status[syslog_mon.raw_severity()-1]), $host[src], "Syslog: #{msg}","#{date}")
								  $host[src].set_trap_severity(syslog_mon.raw_severity()-1)
							  else
								  fac,sev=extract_pri(pri)
								  map_sev=map_syslog_sev(sev)
							  	  $event_win.add_event(EventWindow::SYSLOG_EVENT_TYPE, get_level_from_status($status[map_sev]), $host[src], "Syslog: #{msg}","#{date}")
								  $host[src].set_trap_severity(map_sev)
							  end
							break
      				  end
						end
					end
				end
			end
		end
    end
  end

end
