#
# here we can find open & save function for the host database on a file named 'gnms_host.db'
#
# David Maciejak
# 26012003
#

require "#{GNMSLIB}"+'/node'
require "#{GNMSLIB}"+'/format'

require 'rexml/document'
require 'rexml/streamlistener'
include REXML

DEFAULTHOSTDBFILE="nodedb.xml"
DEFAULTHOSTDB="#{CONF_DIR}/#{DEFAULTHOSTDBFILE}"

DB_SCHEMA_VERSION = 2.1

#
#	open sqlite database, assign $db global variable
#
def open_db(fic=SQLITE_DB)
	if FileTest.exist?(fic) and (!File.readable?(fic) or !File.writable?(fic))
	      $log.fatal("Wrong permission on  #{fic} need read-write access")
	      exit()
        end
	begin
		$db = SQLite3::Database.new(fic)
		$db.default_synchronous = 'off'
		$db.synchronous = 'off'
	rescue
		$log.fatal("#{fic} database corrupted")
		exit()
	end
end

#
# 	close sqlite database
#
def close_db()
	$db.close
end

#
# set mac addr for a specific nodeid
#
def db_set_mac(nodeid, mac)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET mac=? WHERE id=?")
	stmt.execute(mac, nodeid)
	$db.commit
end

#
# set mask as integer value for a specific nodeid
#
def db_set_mask(nodeid, mask)
	if isInteger(mask)
		$db.transaction
		stmt=$db.prepare("UPDATE node SET netmask=? WHERE id=?")
		stmt.execute(mask, nodeid)
		$db.commit
	end
end

#
# set description for a specific nodeid
#
def db_set_description(nodeid, desc)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET description=? WHERE id=?")
	stmt.execute(desc, nodeid)
	$db.commit
end

#
# set severities for a specific nodeid
#
def db_set_severity(nodeid, st, avail, snmp, cstom, trap, serv, wmi, syslog, jmx)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET status=?, avail_sev=?, snmp_sev=?, custom_sev=?, trap_sev=?, service_sev=?, wmi_sev=?, syslog_sev=?, jmx_sev=? WHERE id=?")
	stmt.execute(st, avail, snmp, cstom, trap, serv, wmi, syslog, jmx, nodeid)
	$db.commit
end

#
# set property type and os for a specific nodeid
#
def db_set_node_type_and_os(nodeid, type, os)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET type=?, os=? WHERE id=?")
	stmt.execute(type, os, nodeid)
	$db.commit
end

#
# set property x and y for a specific nodeid
#
def db_set_node_position(nodeid, x, y)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET x=?, y=? WHERE id=?")
	stmt.execute(x, y, nodeid)
	$db.commit
end

#
# set snmp properties for a specific nodeid
#
def db_set_snmp_parameter(nodeid, snmp_version, snmp_port, snmp_timeout, snmp_retry)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET snmp_version=?, snmp_port=?, snmp_timeout=?, snmp_retry=? WHERE id=?")
	stmt.execute(snmp_version, snmp_port, snmp_timeout, snmp_retry, nodeid)
	$db.commit
end

#
# set node size for a specific nodeid
#
def db_set_node_size(nodeid, node_size)
	if isInteger(node_size)
		$db.transaction
		stmt=$db.prepare("UPDATE node SET node_size=? WHERE id=?")
		stmt.execute(node_size, nodeid)
		$db.commit
	end
end

#
# set background conf for a specific nodeid
#
def db_set_background(nodeid, bg_type, bg_data)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET map_bg_type=?, map_bg_data=? WHERE id=?")
	stmt.execute(bg_type, "#{bg_data}", nodeid)
	$db.commit
end

#
# set map for a specific nodeid
#
def db_set_map(nodeid, map_ip)
	#test if we are trying to set map for the root network
	if map_ip
		mapnodeid=db_get_node_id(map_ip,NETWORKNODE)
		if mapnodeid != 0
			$db.transaction
			stmt=$db.prepare("UPDATE node SET map=? WHERE id=?")
			stmt.execute(mapnodeid, nodeid)
			$db.commit
		else
			$log.error("db_set_map: can't find network map #{map_ip}")		
		end
	end
end

#
# set names for a specific nodeid depending of its type
#
def db_set_name(nodeid, name, dnsn, dnsd, netbiosn, netbiosd)
	$db.transaction
	stmt=$db.prepare("UPDATE node SET name=?, dns_name=?, dns_domain=?, netbios_name=?, netbios_domain=? WHERE id=?")
	stmt.execute(name, dnsn, dnsd, netbiosn, netbiosd, nodeid)
	$db.commit
end

#
#	return the service id associated to the protocol/number
#
def db_get_service_id(protocol, number)

	stmt=$db.prepare("SELECT id FROM service WHERE protocol=? and port=?")
 	stmt.execute(protocol,number) do |rs|
		rs.each do |field,type|
			if !field
				return 0
			end
			return field
		end
	end
	return 0
end

def db_get_node_id(ip, ntype)

	stmt=$db.prepare("SELECT id FROM node WHERE ip=? and nodetype=?")
 	stmt.execute(ip, ntype) do |rs|
		rs.each do |field,type|
			if !field
			   return 0
			end
			   return field
		end
	end
	return 0
end

def db_get_node_template_monitor_id(templ, nid)

	stmt=$db.prepare("SELECT node_and_#{templ}_monitor_id FROM node WHERE id=?")
 	stmt.execute(nid) do |rs|
		rs.each do |field,type|
			if !field
				return 0
			end
			return field
		end
	end
	return 0
end

def db_get_node_account_id(nid)

	stmt=$db.prepare("SELECT node_and_account_id FROM node WHERE id=?")
 	stmt.execute(nid) do |rs|
		rs.each do |field,type|
			if !field
				return 0
			end
			return field
		end
	end
	return 0
end

def db_get_node_interface_id(nid)

	stmt=$db.prepare("SELECT node_and_interface_id FROM node WHERE id=?")
 	stmt.execute(nid) do |rs|
		rs.each do |field,type|
			if !field
				return 0
			end
			return field
		end
	end
	return 0
end

def db_get_node_snmp_monitor_id(nid)
	return db_get_node_template_monitor_id("snmp", nid)
end

def db_get_node_wmi_monitor_id(nid)
	return db_get_node_template_monitor_id("wmi", nid)
end

def db_get_node_jmx_monitor_id(nid)
	return db_get_node_template_monitor_id("jmx", nid)
end

def db_get_node_snmptrap_monitor_id(nid)
	return db_get_node_template_monitor_id("snmptrap", nid)
end

def db_get_node_syslog_monitor_id(nid)
	return db_get_node_template_monitor_id("syslog", nid)
end

def db_get_node_custom_monitor_id(nid)
	return db_get_node_template_monitor_id("custom", nid)
end

def db_get_accounts(ip, ntype)
	haccount=Hash.new
	stmt=$db.prepare("SELECT id, node_and_account_id FROM node WHERE ip=? and nodetype=?")
	stmt.execute(ip, ntype) do |rs|
	      rs.each do |nodeid, naccountid|
		      if naccountid
			stmt2=$db.prepare("SELECT account_id FROM node_and_account WHERE node_id=? and id=?")
	   		stmt2.execute(nodeid, naccountid) do |rs|
	      			rs.each do |accountid|
					stmt3=$db.prepare("SELECT idname, login, password  FROM account WHERE id=?")
					stmt3.execute(accountid) do |rs|
						rs.each do |idn, lg, pass|
							#fill the hash
							haccount[idn]=[lg, Fog.decrypt(Base64.decode64(pass),GNMSKEY)]
						end
					end
				end
			
			end
		      end
	      end
	 end
	return haccount
end

def db_get_interfaces(nodeid, nodeandinterface_id)
	haccount=Hash.new
	stmt2=$db.prepare("SELECT interface_id FROM node_and_interface WHERE node_id=? and id=?")
	stmt2.execute(nodeid, nodeandinterface_id) do |rs|
	  rs.each do |interfaceid|
		stmt3=$db.prepare("SELECT name, link_status, speed FROM interface WHERE id=?")
		stmt3.execute(interfaceid) do |rs|
			rs.each do |n, lstatus, sp|
			#fill the hash
			haccount[n]=Interface.new(n, lstatus, sp)
			end
		end
	  end
	end
	return haccount
end

#
# at db creation, populate the custom_monitor table
# with plugins found
#
def db_set_custom_monitor()
  local_custom_monitor = []
  begin
  stmt1=$db.prepare("SELECT custom_name FROM custom_monitor")
	stmt1.execute() do |rs|
    rs.each do |cm_name|
      local_custom_monitor.push cm_name.to_s
    end
  end
  rescue Exception => msg
    $log.error("db_set_custom_monitor: #{msg}")
    return
  end
  if $custom_monitor
    stmt2=$db.prepare("INSERT INTO custom_monitor VALUES (NULL, ?)")
    $custom_monitor.each do |cm_name|
      #check if cm_name is not in the db
      if local_custom_monitor.index(cm_name).nil?
          $db.transaction
		stmt2.execute(cm_name)
          $db.commit
      end
    end
  end
end

#
#	add a custom monitoring for specific node
#
def db_add_custom_monitoring(ip, nodetype, custom_mon_array)
  #here custom_monitor table is up to date
	nodeid=db_get_node_id(ip, nodetype)
	nodecustommonitoringid=db_get_node_custom_monitor_id(nodeid)
	if nodecustommonitoringid == 0
			   stmt=$db.prepare("UPDATE node SET node_and_custom_monitor_id=(SELECT IFNULL(MAX(id),0) FROM node_and_custom_monitor)+1 WHERE id=?")
				$db.transaction
				stmt.execute(nodeid)
				$db.commit
				nodecustommonitoringid=db_get_node_custom_monitor_id(nodeid)
	end

	if nodecustommonitoringid != 0
  	  stmt2=$db.prepare("SELECT id FROM custom_monitor WHERE custom_name = ?")
	  stmt4=$db.prepare("INSERT INTO node_and_custom_monitor VALUES (?, ?, ?, ?, ?, ?)")
	  $db.transaction
	  custom_mon_array.each {|state, name, param, sev|
		stmt2.execute(name) do |rs|		
			rs.each do |customnonid|
				stmt4.execute(nodecustommonitoringid, nodeid, customnonid, state, param, sev)
			end
		end
	  }
	  $db.commit
	else
		$log.error("db_add_custom_monitoring: something goes wrong trying to add the custom monitoring id")
		return
	end
end

def db_add_service(ip, nodetype, proto, number, name, monitor_flag, sev)
	serv_id=db_get_service_id(proto, number)
	if serv_id == 0
		stmt=$db.prepare("INSERT INTO service VALUES (NULL, ?, ?, ?)")
		$db.transaction
		stmt.execute(proto, number, name)
		$db.commit
		serv_id=db_get_service_id(proto, number)
	end

	#add the monitoring in service_monitor table
	if serv_id != 0
	   #request service monitor id	
	   stmt=$db.prepare("SELECT id, service_monitor_id FROM node WHERE ip=? and nodetype=?")
	   stmt.execute(ip, nodetype) do |rs|
	      rs.each do |hid,smid|
		      if !smid
			      stmt2=$db.prepare("INSERT INTO service_monitor VALUES ((SELECT IFNULL(MAX(id),0) FROM service_monitor)+1, ?, ?, ?, ?)")
			      $db.transaction
			      begin
				      stmt2.execute(hid, serv_id, monitor_flag, sev)
			      rescue
				      $log.error("error inserting in db_add_service #{hid} #{serv_id} #{monitor_flag} #{sev}")
			      end
					

			      stmt3=$db.prepare("SELECT distinct(id) FROM service_monitor WHERE node_id=? and service_id=?")
			      stmt3.execute(hid, serv_id) do |rs|
				      rs.each do |field,type|
					      stmt4=$db.prepare("UPDATE node SET service_monitor_id=? WHERE id=?")
					      begin
					      	stmt4.execute(field, hid)
					      rescue
				  	        $log.error("error updating in db_add_service #{field} #{hid}")
					      end
				      end		
			      end
			      $db.commit
		      else
			      stmt2=$db.prepare("INSERT INTO service_monitor VALUES (?, ?, ?, ?, ?)")
				begin
			      $db.transaction		
			      stmt2.execute(smid, hid, serv_id, monitor_flag, sev)
			      $db.commit
				rescue
				  $log.error("error inserting in db_add_service #{smid} #{hid} #{serv_id} #{monitor_flag} #{sev}")
				end
		      end
	      end
	   end
	else
		$log.error("db_add_service: something goes wrong trying to find the service id")
	end
end

def db_add_snmp_monitoring(ip, nodetype, state, ref_name, op, val, sev)
	db_add_template_monitoring("snmp", ip, nodetype, state, ref_name, op, val, sev)
end

def db_add_wmi_monitoring(ip, nodetype, state, ref_name, op, val, sev)
	db_add_template_monitoring("wmi", ip, nodetype, state, ref_name, op, val, sev)
end

def db_add_snmptrap_monitoring(ip, nodetype, state, ref_name, sev)
	db_add_template_monitoring("snmptrap", ip, nodetype, state, ref_name, sev)
end

def db_add_syslog_monitoring(ip, nodetype, state, ref_name, sev)
	db_add_template_monitoring("syslog", ip, nodetype, state, ref_name, sev)
end

def db_add_jmx_monitoring(ip, nodetype, state, ref_name, proto, port, auth, attr_h)
	nodeid=db_get_node_id(ip, nodetype)
	nodejmxmonitoringid=db_get_node_jmx_monitor_id(nodeid)
	if nodejmxmonitoringid == 0
		stmt=$db.prepare("UPDATE node SET node_and_jmx_monitor_id=(SELECT IFNULL(MAX(id),0) FROM node_and_jmx_monitor)+1 WHERE id=?")
		$db.transaction
		stmt.execute(nodeid)
		$db.commit
		nodejmxmonitoringid=db_get_node_jmx_monitor_id(nodeid)
	end

	if nodejmxmonitoringid != 0
		stmt2=$db.prepare("SELECT IFNULL(MAX(id),0)+1 FROM jmx_monitor")
		stmt2.execute do |rs|
			rs.each do |jmxmonid|
				$db.transaction
				stmt3=$db.prepare("INSERT INTO jmx_monitor VALUES (?, ?, ?, ?, ?, ?)")
				stmt3.execute(jmxmonid, state, ref_name, proto, port, auth)
				
				stmt4=$db.prepare("SELECT IFNULL(MAX(id),0)+1 FROM jmx_attr_monitor")
				stmt4.execute do |rs|
					rs.each do |jmxattrmonid|
						stmt5=$db.prepare("INSERT INTO jmx_attr_monitor VALUES (?, ?, ?, ?, ?)")
						attr_h.each_value {|attr|
							stmt5.execute(jmxattrmonid, attr.attributeName, attr.operator, attr.value, attr.severity)
						}
						stmt6=$db.prepare("INSERT INTO node_and_jmx_monitor VALUES (?, ?, ?, ?)")
						stmt6.execute(nodejmxmonitoringid, nodeid, jmxmonid, jmxattrmonid)
						$db.commit
					end
				end
			end
		end
	else
		$log.error("db_add_jmx_monitoring: something goes wrong trying to find the jmx monitoring id")
		return
	end
end

#
# template to add a monitoring to a specific node
#
def db_add_template_monitoring(templ, ip, nodetype, state, oidref_name, op, val=nil, sev=nil)
	nodeid=db_get_node_id(ip, nodetype)
	nodesnmpmonitoringid=eval "db_get_node_#{templ}_monitor_id(nodeid)"
	if nodesnmpmonitoringid == 0
			   stmt=$db.prepare("UPDATE node SET node_and_#{templ}_monitor_id=(SELECT IFNULL(MAX(id),0) FROM node_and_#{templ}_monitor)+1 WHERE id=?")
				$db.transaction
				stmt.execute(nodeid)
				$db.commit
				nodesnmpmonitoringid=eval "db_get_node_#{templ}_monitor_id(nodeid)"
	end

	if nodesnmpmonitoringid != 0
		stmt2=$db.prepare("SELECT IFNULL(MAX(id),0)+1 FROM #{templ}_monitor")
		stmt2.execute do |rs|
			rs.each do |snmpmonid|
				$db.transaction
				if !sev
					#we are in snmptrap or syslog monitoring
					stmt3=$db.prepare("INSERT INTO #{templ}_monitor VALUES (?, ?, ?, ?)")
					stmt3.execute(snmpmonid, state, oidref_name, op)
				else
					stmt3=$db.prepare("INSERT INTO #{templ}_monitor VALUES (?, ?, ?, ?, ?, ?)")
					stmt3.execute(snmpmonid, state, oidref_name, op, val, sev)
				end
				stmt4=$db.prepare("INSERT INTO node_and_#{templ}_monitor VALUES (?, ?, ?)")
				stmt4.execute(nodesnmpmonitoringid, nodeid, snmpmonid)
				$db.commit
			end
		end
	else
		$log.error("db_add_#{templ}_monitoring: something goes wrong trying to find the #{templ} monitoring id")
		return
	end
end

#
# add an account to a specific node
#
def db_add_account(ip, nodetype, accounts)
	nodeid=db_get_node_id(ip, nodetype)
	nodeaccountid=db_get_node_account_id(nodeid)
	if nodeaccountid == 0
			stmt=$db.prepare("UPDATE node SET node_and_account_id=(SELECT IFNULL(MAX(id),0) FROM node_and_account)+1 WHERE id=?")
			begin
				$db.transaction
				stmt.execute(nodeid)
				$db.commit
			rescue
				$log.error("error updating node_and_account_id table")
			end
			nodeaccountid=db_get_node_account_id(nodeid)
	end
	if nodeaccountid != 0
		account_id = 1
		stmt2=$db.prepare("SELECT IFNULL(MAX(id),0)+1 AS cid FROM account")		
		rs = stmt2.execute
		account_id =  rs.first.join("\t").to_s.to_i
		stmt3=$db.prepare("INSERT INTO account VALUES (?, ?, ?, ?)")
		stmt4=$db.prepare("INSERT INTO node_and_account VALUES (?, ?, ?)")
	        $db.transaction
	        accounts.each { |idname, login, passwd|
	          password=Base64.encode64(Fog.new(passwd,GNMSKEY))
		  stmt3.execute(account_id, idname, login, password)
		  stmt4.execute(nodeaccountid, nodeid, account_id)
		  account_id +=1
		}
		$db.commit
	else
		$log.error("db_add_account: something goes wrong trying to find the account id")
		return
	end
end


#
# add an interface to a specific node
#
def db_add_interface(ip, nodetype, name, lstatus, sp, tp)
	nodeid=db_get_node_id(ip, nodetype)
	nodeinterfaceid=db_get_node_interface_id(nodeid)
	if nodeinterfaceid == 0
			stmt=$db.prepare("UPDATE node SET node_and_interface_id=(SELECT IFNULL(MAX(id),0) FROM node_and_interface)+1 WHERE id=?")
			begin
				$db.transaction
				stmt.execute(nodeid)
				$db.commit
			rescue
				$log.error("error updating node_and_interface_id table")
			end
			nodeinterfaceid=db_get_node_interface_id(nodeid)
	end

	if nodeinterfaceid != 0
		stmt2=$db.prepare("SELECT IFNULL(MAX(id),0)+1 FROM interface")
		stmt2.execute do |rs|
			rs.each do |interfaceid|
				$db.transaction
				stmt3=$db.prepare("INSERT INTO interface VALUES (?, ?, ?, ?, ?)")
				stmt3.execute(interfaceid, name, lstatus, sp, tp)
				stmt4=$db.prepare("INSERT INTO node_and_interface VALUES (?, ?, ?)")
				stmt4.execute(nodeinterfaceid, nodeid, interfaceid)
				$db.commit
			end
		end			
	else
		$log.error("db_add_interface: something goes wrong trying to find the interface id")
		return
	end	
end

#
# set an account to a specific node
#
def db_set_account(nodeid, idname, login, passwd)
	nodeaccountid=db_get_node_account_id(nodeid)
	password=Base64.encode64(Fog.new(passwd,GNMSKEY))
	if nodeaccountid == 0
			   stmt=$db.prepare("UPDATE node SET node_and_account_id=(SELECT IFNULL(MAX(id),0) FROM node_and_account)+1 WHERE id=?")
				$db.transaction
				stmt.execute(nodeid)
				$db.commit
				nodeaccountid=db_get_node_account_id(nodeid)
	end

	if nodeaccountid != 0
		stmt2=$db.prepare("SELECT account_id FROM node_and_account WHERE node_id=? and id=?")
		stmt2.execute(nodeid, nodeaccountid) do |rs|
			rs.each do |accountid|
				$db.transaction
				stmt3=$db.prepare("UPDATE account SET login=?,password=? WHERE id=? and idname=?")
				stmt3.execute(login, password, accountid, idname)
				$db.commit
				return
			end
		end			
	else
		$log.error("db_set_account: something goes wrong trying to find the account id")
		return
	end
end

#
# set interfaces to a specific node
#
def db_set_interfaces(ip, interfaces)
        #removing entries from the db
	db_del_interfaces(ip, HOSTNODE)
	#saving interfaces in db
	interfaces.each_value {|intf|
	   db_add_interface(ip, HOSTNODE, intf.name, intf.link_status, intf.speed, intf.type)
	}
end

#
# remove all service monitoring for a specific node
#
def db_del_services(node_ip, node_type)
	stmt=$db.prepare("SELECT id, service_monitor_id FROM node WHERE ip=? and nodetype=?")
 	stmt.execute(node_ip,node_type) do |rs|
		rs.each do |nodeid,servicemid|
			#check if we already monitor service
			if servicemid
				stmt=$db.prepare("DELETE from service_monitor WHERE node_id=? and id=?")
				$db.transaction
		 		stmt.execute(nodeid, servicemid)
				$db.commit
			end	
		end
	end
end

#
# remove all custom monitoring for a specific node
#
def db_del_custom_monitoring(node_ip, node_type)
stmt=$db.prepare("SELECT id, node_and_custom_monitor_id FROM node WHERE ip=? and nodetype=?")
 	stmt.execute(node_ip,node_type) do |rs|
		rs.each do |nodeid,nodesnmpmonid|
     			$db.transaction
					stmt2=$db.prepare("DELETE from node_and_custom_monitor WHERE node_id=? and id=?")
			 		stmt2.execute(nodeid,nodesnmpmonid)
					$db.commit
		end
	end
end

#
# remove all monitoring stuff based on a given monitoring template for a specific node
#
def db_del_template_monitoring(templ, node_ip, node_type)
	stmt=$db.prepare("SELECT id, node_and_#{templ}_monitor_id FROM node WHERE ip=? and nodetype=?")
 	stmt.execute(node_ip,node_type) do |rs|
		rs.each do |nodeid,nodesnmpmonid|
			if nodesnmpmonid
				stmt=$db.prepare("SELECT distinct #{templ}_monitor_id FROM node_and_#{templ}_monitor WHERE node_id=? and id=?")
		 		stmt.execute(nodeid, nodesnmpmonid) do |rs|
					rs.each do |snmpmonid|
						if snmpmonid
							stmt=$db.prepare("DELETE from #{templ}_monitor WHERE id=?")
	      				$db.transaction
					 		stmt.execute(snmpmonid)
							$db.commit
						end
					end
     				$db.transaction
					stmt2=$db.prepare("DELETE from node_and_#{templ}_monitor WHERE node_id=? and id=?")
			 		stmt2.execute(nodeid,nodesnmpmonid)
					$db.commit
				end
			end	
		end
	end
end

#
# remove all accounts stuff for a specific node
#
def db_del_accounts(node_ip, node_type)
	stmt=$db.prepare("SELECT id, node_and_account_id FROM node WHERE ip=? and nodetype=?")
 	stmt.execute(node_ip,node_type) do |rs|
		rs.each do |nodeid,nodeaccountid|
			if nodeaccountid
				stmt=$db.prepare("SELECT distinct account_id FROM node_and_account WHERE node_id=? and id=?")
		 		stmt.execute(nodeid, nodeaccountid) do |rs|
					rs.each do |accountid|
						if accountid
							stmt2=$db.prepare("DELETE from account WHERE id=?")
			      				$db.transaction
					 		stmt2.execute(accountid)
							$db.commit
						end
					end
					stmt3=$db.prepare("DELETE from node_and_account WHERE node_id=? and id=?")
			      		$db.transaction
					stmt3.execute(nodeid,nodeaccountid)
					$db.commit										
				end
			end	
		end
	end
end

#
# remove all interfaces stuff for a specific node
#
def db_del_interfaces(node_ip, node_type)
	stmt=$db.prepare("SELECT id, node_and_interface_id FROM node WHERE ip=? and nodetype=?")
 	stmt.execute(node_ip,node_type) do |rs|
		rs.each do |nodeid,nodeinterfaceid|
			if nodeinterfaceid
				stmt=$db.prepare("SELECT distinct interface_id FROM node_and_interface WHERE node_id=? and id=?")
		 		stmt.execute(nodeid, nodeinterfaceid) do |rs|
					rs.each do |interfaceid|
						if interfaceid
							stmt2=$db.prepare("DELETE from interface WHERE id=?")
			      				$db.transaction
					 		stmt2.execute(interfaceid)
							$db.commit
						end
					end
					stmt3=$db.prepare("DELETE from node_and_interface WHERE node_id=? and id=?")
			      		$db.transaction
					stmt3.execute(nodeid,nodeinterfaceid)
					$db.commit										
				end
			end	
		end
	end
end

#
# remove all groups stuff for a specific node
#
def db_del_groups(node_ip, node_type)

end


#
#	return an array of Contact_selected_info for a given id
#
def db_get_alert_override_contact_array(id)
	contactt=Array.new
	stmt=$db.prepare("SELECT contact_id FROM alert_override_contact WHERE id=?")
	stmt.execute(id) do |rs|
		rs.each do |cid|
			if cid
				(firstname, lastname)=db_get_contact_names(cid)
				contactt.push(Contact_selected_info.new(firstname,lastname))
			end
		end
	end
	return contactt
end

#
#	get alert override
#
def db_get_alert_override(node, node_type)
	nid=db_get_node_id(node.ip, node_type)
	stmt=$db.prepare("SELECT alert_override_detail_id, alert_type_by_level FROM alert_override WHERE node_id=?")
	stmt.execute(nid) do |rs|
			cpt=0
			rs.each do |aodid, atbl|				
				cpt+=1
				stmt2=$db.prepare("SELECT mail_state, mail_by_level_id, popup_state, beep_state, sound_file_path_by_level, syslog_state,sms_state, sms_num_by_level_id, im_state,im_dest_by_level_id, script_state, script_name_by_level, irc_state FROM alert_override_detail WHERE id=?")
				stmt2.execute(aodid) do |rs|
					rs.each do |mail_state, mail_by_level_id, popup_state, beep_state, sound_file_path_by_level, syslog_state,sms_state, sms_num_by_level_id, im_state,im_dest_by_level_id, script_state, script_name_by_level, irc_state|
						node.alert_type_by_level[atbl][0]=mail_state.to_b
						node.mail_by_level[atbl]=db_get_alert_override_contact_array(mail_by_level_id)
						node.alert_type_by_level[atbl][1]=popup_state.to_b
						node.alert_type_by_level[atbl][2]=beep_state.to_b
						node.sound_file_path_by_level[atbl]=sound_file_path_by_level
						node.alert_type_by_level[atbl][3]=syslog_state.to_b
						node.alert_type_by_level[atbl][4]=sms_state.to_b
						node.sms_num_by_level[atbl]=db_get_alert_override_contact_array(sms_num_by_level_id)
						node.alert_type_by_level[atbl][5]=im_state.to_b
						node.im_dest_by_level[atbl]=db_get_alert_override_contact_array(im_dest_by_level_id)
						node.alert_type_by_level[atbl][6]=script_state.to_b
						node.script_name_by_level[atbl]=script_name_by_level												
						node.alert_type_by_level[atbl][7]=irc_state.to_b
					end
				end
			end	
			
			if cpt > 0
				node.set_inherit_conf(false)
			end
	end
end

#
# set alert override for a specific node
#
def db_set_alert_override(node, node_type)
	nid=db_get_node_id(node.ip, node_type)
	$LEVEL.each {|lvl|
		mail_id=nil
		sms_id=nil
		im_id=nil
		stmt=$db.prepare("SELECT IFNULL(MAX(id),0)+1 FROM alert_override_contact")
		stmt2=$db.prepare("INSERT INTO alert_override_contact VALUES (?,?)")
		stmt.execute() do |rs|
			rs.each do |maxid,type|
				id=maxid.to_i
				
				if node.mail_by_level[lvl]
				node.mail_by_level[lvl].each {|contact|
						cid=db_get_contact_id(contact.firstname, contact.lastname)
						if cid
							stmt2.execute(id, cid)
						end
					}
					mail_id=id
					id+=1
				else
					maid_id=nil
				end
				if node.sms_num_by_level[lvl]
				node.sms_num_by_level[lvl].each {|contact|
						cid=db_get_contact_id(contact.firstname, contact.lastname)
						if cid
							stmt2.execute(id, cid)
						end
					}
					sms_id=id
					id+=1
				else
					sms_id=nil
				end
				if node.im_dest_by_level[lvl]
				node.im_dest_by_level[lvl].each {|contact|
						cid=db_get_contact_id(contact.firstname, contact.lastname)
						if cid
							stmt2.execute(id, cid)
						end
					}
					im_id=id
					id+=1
				else
					im_id=nil
				end
			end
		end
	
		stmt=$db.prepare("INSERT INTO alert_override_detail VALUES (NULL, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
		$db.transaction
		stmt.execute(node.alert_type_by_level[lvl][0], mail_id, node.alert_type_by_level[lvl][1], node.alert_type_by_level[lvl][2],node.sound_file_path_by_level[lvl],node.alert_type_by_level[lvl][3],node.alert_type_by_level[lvl][4],sms_id,node.alert_type_by_level[lvl][5],im_id,node.alert_type_by_level[lvl][6],node.script_name_by_level[lvl])
		$db.commit
		stmt=$db.prepare("SELECT MAX(id) FROM alert_override_detail")
		stmt.execute() do |rs|
			rs.each do |aodid|
				if aodid
					stmt2=$db.prepare("INSERT INTO alert_override VALUES (NULL,?,?,?)")
					$db.transaction
					stmt2.execute(nid, aodid, lvl)
					$db.commit
				end
			end
		end
	}

end

#
# remove all alert override for a specific node
#
def db_del_alert_override(node_ip, node_type=nil)
	if !node_type
		node_type=NETWORKNODE
	end
	nid=db_get_node_id(node_ip, node_type)
	stmt=$db.prepare("SELECT alert_override_detail_id FROM alert_override WHERE node_id=?")
	stmt.execute(nid) do |rs|
		rs.each do |aoid|
			
			stmt2=$db.prepare("SELECT mail_by_level_id, sms_num_by_level_id, im_dest_by_level_id FROM alert_override_detail WHERE id=?")
			stmt2.execute(aoid) do |rs|
				rs.each do |mail_contact_id, sms_contact_id, im_contact_id|
					stmt3=$db.prepare("DELETE from alert_override_contact WHERE id=?")
					$db.transaction
					stmt3.execute(mail_contact_id)
					stmt3.execute(sms_contact_id)
					stmt3.execute(im_contact_id)
					$db.commit
				end
			end			
			stmt4=$db.prepare("DELETE from alert_override_detail WHERE id=?")
			$db.transaction
			stmt4.execute(aoid)
			$db.commit
		end
	end
	stmt5=$db.prepare("DELETE from alert_override WHERE node_id=?")
	$db.transaction
	stmt5.execute(nid)
	$db.commit	
end

#
# remove all links stuff for a specific node
#
def db_del_links(node_ip=nil, node_type=nil)
	if !node_ip
		stmt=$db.prepare("DELETE from link")
		$db.transaction
		stmt.execute()
		$db.commit
	else
		nid=db_get_node_id(node_ip, node_type)
		stmt=$db.prepare("DELETE from link WHERE node1=? or node2=?")
		$db.transaction
		stmt.execute(nid,nid)
		$db.commit										
	end
end

#
#	call Link class and fill directly the $links
#
def db_get_links()
	stmt=$db.prepare("SELECT link_state, name, speed, node1, node2 FROM link")
 	stmt.execute() do |rs|
		rs.each do |link_state, name, speed, node1, node2|
			ip1, nodetype1 = db_get_ip_type(node1)
			ip2, nodetype2 = db_get_ip_type(node2)
			if nodetype1 == HOSTNODE
				node1=$host[ip1]
			else
				node1=$network[ip1]
			end
			if nodetype2 == HOSTNODE
				node2=$host[ip2]
			else
				node2=$network[ip2]
			end
			Link.new(node1, node2)
		end
	end
end

#
#	save link for a given node, this method save the $links array to db
#
def db_set_links()
	$links.each {|link|
		node_ip1=link.node1.ip
		node_ip2=link.node2.ip
		node_type1=HOSTNODE
		if link.node1.class == Network
			node_type1=NETWORKNODE
		end
		node_type2=HOSTNODE
		if link.node2.class == Network
			node_type2=NETWORKNODE
		end	
		nid1=db_get_node_id(node_ip1, node_type1)
		nid2=db_get_node_id(node_ip2, node_type2)

      		stmt=$db.prepare("INSERT INTO link VALUES (?, ?, ?, ?, ?, ?)")
		$db.transaction
		stmt.execute(nil, 1, "", 0, nid1, nid2)
		$db.commit
	}
end

#
# remove all snmp monitoring stuff for a specific node
#
def db_del_snmp_monitoring(node_ip, node_type)
	db_del_template_monitoring("snmp", node_ip, node_type)
end

#
# remove all wmi monitoring stuff for a specific node
#
def db_del_wmi_monitoring(node_ip, node_type)
	db_del_template_monitoring("wmi", node_ip, node_type)
end

#
# remove all jmx monitoring stuff for a specific node
#
def db_del_jmx_monitoring(node_ip, node_type)
	stmt=$db.prepare("SELECT id, node_and_jmx_monitor_id FROM node WHERE ip=? and nodetype=?")
 	stmt.execute(node_ip,node_type) do |rs|
		rs.each do |nodeid,nodejmxid|
			if nodejmxid
				stmt=$db.prepare("SELECT jmx_monitor_id, jmx_attr_monitor_id FROM node_and_jmx_monitor WHERE node_id=? and id=?")
		 		stmt.execute(nodeid, nodejmxid) do |rs|
					rs.each do |jmxmonid, jmxattrmonid|
						if jmxmonid
							#we can pass here multiple time
							stmt=$db.prepare("DELETE from jmx_monitor WHERE id=?")
	      						$db.transaction
					 		stmt.execute(jmxmonid)
							$db.commit
						end
						if jmxattrmonid
							stmt=$db.prepare("DELETE from jmx_attr_monitor WHERE id=?")
	      						$db.transaction
					 		stmt.execute(jmxattrmonid)
							$db.commit
						end
					end
     					$db.transaction
					stmt2=$db.prepare("DELETE from node_and_jmx_monitor WHERE node_id=? and id=?")
			 		stmt2.execute(nodeid,nodejmxid)
					$db.commit
				end
			end	
		end
	end
end

#
# remove all snmp trap monitoring stuff for a specific node
#
def db_del_snmptrap_monitoring(node_ip, node_type)
	db_del_template_monitoring("snmptrap", node_ip, node_type)
end

#
# remove all syslog monitoring stuff for a specific node
#
def db_del_syslog_monitoring(node_ip, node_type)
	db_del_template_monitoring("syslog", node_ip, node_type)
end


def db_select_all_prp(table)
	return $db.prepare("SELECT * FROM #{table}")
end

def db_delete_all_prp(table)
	begin
		return $db.prepare("DELETE FROM #{table}")
	rescue Exception => msg
		$log.info("Error when trying to delete table: #{msg}")
		return nil
	end
end

def db_insert_ref_prp4(table)
	return $db.prepare("INSERT INTO #{table} VALUES (?, ?, ?, ?)")
end

def db_insert_ref_prp(table)
	return $db.prepare("INSERT INTO #{table} VALUES (?, ?, ?)")
end

def db_delete_all_node_prp()
	return $db.prepare("DELETE FROM node")
end

def db_insert_custom_monitor(cm_name_id)
	stmt=$db.prepare("INSERT INTO custom_monitor VALUES (NULL,?)")
	$db.transaction
 	stmt.execute(cm_name_id)
	$db.commit
end

def db_get_custom_monitor_id(cm_name_id)
	stmt=$db.prepare("SELECT id FROM custom_monitor WHERE name=?")
 	stmt.execute(cm_name_id) do |rs|
		rs.each do |field,type|
			if !field
				return 0
			end
			return field.to_i
		end
	end
	return 0
end

def db_get_network_id(map)
	stmt=$db.prepare("SELECT id FROM node WHERE nodetype=? and ip=?")
 	stmt.execute(NETWORKNODE, map) do |rs|
		rs.each do |field,type|
			if !field
				return 0
			end
			return field.to_i
		end
	end
	return 0
end

#
#	delete a node from the db and all the stuff around it
#
def db_delete_node(node_type, ip, mask)
	#sqlite does not support cascading delete
	begin
		db_del_services(ip, node_type)
		db_del_snmp_monitoring(ip, node_type)
		db_del_wmi_monitoring(ip, node_type)
		db_del_jmx_monitoring(ip, node_type)
		db_del_snmptrap_monitoring(ip, node_type)
		db_del_syslog_monitoring(ip, node_type)
		db_del_custom_monitoring(ip, node_type)
		db_del_links(ip, node_type)
		db_del_accounts(ip, node_type)
		db_del_interfaces(ip, node_type)
		db_del_groups(ip, node_type)
		db_del_alert_override(ip, node_type)

		$db.transaction
		stmt=$db.prepare("DELETE FROM node WHERE nodetype=? and ip=? and netmask=?")
		stmt.execute(node_type,ip,mask)
		$db.commit
	rescue Exception => msg
		$log.error("Error when trying to delete #{ip}/#{mask} from db #{msg.backtrace.join("\n")}")
	end
end

def db_insert_node_prp(uuid,nodetype,ip,mask,ipv6,mask6,mac,map,status,avail_sev,snmp_sev,custom_sev,trap_sev, serv_sev, wmi_sev, syslog_sev, jmx_sev, x,y,icon,description,snmp_version,snmp_port,snmp_timeout,snmp_retry,type,os,name,dns_name,dns_domain,netbios_name,netbios_domain,nodesize,bg_type,bg_data)
	  if defined?($db) && ($db !=nil)
		stmt=$db.prepare("SELECT count(id) FROM node WHERE nodetype=? and ip=? and netmask=?")
	 	stmt.execute(nodetype,ip,mask) do |rs|
			rs.each do |field,vtype|
				if field.to_i==0
				  map_id=db_get_network_id(map)
				  begin
					  $db.transaction
					  stmt=$db.prepare("INSERT INTO node VALUES (NULL,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)")
					  stmt.execute(uuid,nodetype,ip,mask,ipv6,mask6,mac,map_id,status,avail_sev,snmp_sev,custom_sev,trap_sev, serv_sev, wmi_sev, syslog_sev, jmx_sev,x,y,icon,description,snmp_version,snmp_port,snmp_timeout,snmp_retry,type,os,name,dns_name,dns_domain,netbios_name,netbios_domain,nodesize,bg_type,bg_data)
					  $db.commit
				  rescue
				  	$log.error("Error when adding a new node in database")
				  end
				end
			end
		end
	  end
end

#
#	return the ip corresponding to a given node id
#
def db_get_ip(nid)
	if nid
		stmt=$db.prepare("SELECT ip FROM node WHERE id=?")
 		stmt.execute(nid) do |rs|
			rs.each do |field,type|
				if !field
					return nil
				end
				return field
			end
		end
	end
	return nil		
end

#
#	return an array of ip and type corresponding to a given node id
#
def db_get_ip_type(nid)
	if nid
		stmt=$db.prepare("SELECT ip, nodetype FROM node WHERE id=?")
 		stmt.execute(nid) do |rs|
			rs.each do |ip,type|
				return [ip, type]
			end
		end
	end
	return nil		
end

#
#	return a hash of all services for a given node id and service monitor id
#
def db_get_services(nodeid, service_monitor_id)
		if !service_monitor_id		
			return nil
		end
		services=Hash.new
		stmt=$db.prepare("SELECT protocol, port, name, monitor, service_sev FROM service, service_monitor WHERE service_monitor.node_id=? and service_monitor.id=? and service_monitor.service_id=service.id")
		stmt.execute(nodeid, service_monitor_id) do |rs|
			rs.each do |proto, port, name, monitor, sev|
					services["#{proto}/#{port}"]=Service.new(proto, port.to_i, name, monitor, sev.to_i)
			end
		end
		if services.size() == 0
			return nil
		else
			return services
		end
end

def db_get_jmx_monitoring(ip, type)
	stmt=$db.prepare("SELECT id, node_and_jmx_monitor_id FROM node WHERE nodetype=? and ip=?")
	stmt.execute(type, ip) do |rs|
		rs.each do |id, jmxid|
			db_get_jmx_monitoring_ids(ip, id, jmxid)
		end
	end
end

def db_get_jmx_monitoring_ids(ip, id, node_and_jmx_monitor_id)
	stmt=$db.prepare("SELECT distinct jmx_monitor.id, jmx_state, jmx_ref_name, jmx_protocol, jmx_port, jmx_auth FROM node_and_jmx_monitor, jmx_monitor WHERE node_id=? and node_and_jmx_monitor.id=? and jmx_monitor_id=jmx_monitor.id")
	stmt.execute(id, node_and_jmx_monitor_id) do |rs|
	rs.each do |jmx_id, state, jmxref, proto, port, auth|
		attribuesh=Hash.new
		stmt=$db.prepare("SELECT name, oper, value, jmxsev FROM node_and_jmx_monitor, jmx_attr_monitor WHERE node_id=? and node_and_jmx_monitor.id=? and jmx_attr_monitor_id=jmx_attr_monitor.id and node_and_jmx_monitor.jmx_monitor_id=?")
		stmt.execute(id, node_and_jmx_monitor_id, jmx_id) do |rs|
		rs.each do |name, oper, value, jmxsev|
				attribuesh[name]=JmxAttributeMonitoring.new(name, oper.to_i(), value.to_i(), jmxsev.to_i())
			end
		end
		$host[ip].add_jmx_monitoring(state.to_i(), jmxref, proto, port.to_i(), auth, attribuesh.dup)
		end
	end
end

def db_read_node()

	#first read network node details
	#here we want recursive network id, map id is increasingm and 0 is the root map
	stmt=$db.prepare("SELECT * FROM node WHERE nodetype='n' ORDER BY map")
	networkinmap=Hash.new
	stmt.execute() do |rs|
		rs.each do |id, uuid, nodetype, ip, mask, ipv6, mask6, mac, map, status, avail_sev, snmp_sev,custom_sev,trap_sev, serv_sev, wmi_sev, syslog_sev, jmx_sev, service_monitor_id,node_and_snmp_monitor_id,node_and_wmi_monitor_id, node_and_jmx_monitor_id, node_and_snmptrap_monitor_id,node_and_syslog_monitor_id, node_and_custom_monitor_id, node_and_account_id,node_and_group_id, node_and_interface_id, x,y,icon,description, snmp_version,snmp_port, snmp_timeout, snmp_retry,type,os,name,dns_name,dns_domain,netbios_name,netbios_domain,node_size,map_bg_type,map_bg_data|
		
		#read service
		services=db_get_services(id, service_monitor_id)
		map_ip=db_get_ip(map)
		nnode=Network.new(ip, mask.to_i, ipv6, mask6.to_i, map_ip, services, status.to_i, x.to_i, y.to_i, icon, description, type, os, name, dns_domain, netbios_domain, uuid)
		nnode.snmp_version=snmp_version
		nnode.snmp_port=snmp_port
		nnode.snmp_timeout=snmp_timeout
		nnode.snmp_retry=snmp_retry
		
		if node_size.to_i() == 0
			nnode.node_size=NodeView::NODE_SIZE
		else
			nnode.node_size=node_size.to_i()
		end
		
		nnode.map_bg_type=map_bg_type
		if map_bg_type == "color"
			#rgb is store as a text space char separated
			nnode.map_bg_data=map_bg_data.split
		else
			nnode.map_bg_data=map_bg_data
		end
		nnode.set_all_account(db_get_accounts(ip, nodetype))
		
		#load alert override
		db_get_alert_override(nnode, NETWORKNODE)
		
		nnode.reload
		networkinmap[ip]=map_ip
		$network[ip]=nnode
		end
	end
	networkinmap.each_pair do |network_ip, map|
		$network[map].add_network($network[network_ip]) unless map == nil
	end	
	
	#then read the other nodes
	stmt=$db.prepare("SELECT * FROM node WHERE nodetype!='n' ORDER BY ip")
		stmt.execute() do |rs|
			rs.each do |id, uuid, nodetype, ip, mask, ipv6, mask6, mac, map, status, avail_sev, snmp_sev,custom_sev,trap_sev, serv_sev, wmi_sev, syslog_sev, jmx_sev, service_monitor_id,node_and_snmp_monitor_id,node_and_wmi_monitor_id, node_and_jmx_monitor_id, node_and_snmptrap_monitor_id,node_and_syslog_monitor_id, node_and_custom_monitor_id,node_and_account_id,node_and_group_id, node_and_interface_id, x,y,icon,description, snmp_version,snmp_port, snmp_timeout, snmp_retry,type,os,name,dns_name,dns_domain,netbios_name,netbios_domain,node_size,map_bg_type,map_bg_data|
			
			#read service
			services=db_get_services(id, service_monitor_id)
			map_ip=db_get_ip(map)
			nnode=Host.new(ip, mask.to_i, ipv6, mask6.to_i, map_ip, mac, dns_name, netbios_name, services, status.to_i, x.to_i, y.to_i, icon, description, type, os, uuid)
			nnode.set_severities(avail_sev.to_i, snmp_sev.to_i, custom_sev.to_i, trap_sev.to_i,serv_sev.to_i, wmi_sev.to_i, syslog_sev.to_i, jmx_sev.to_i)
			nnode.set_all_account(db_get_accounts(ip, nodetype))
			nnode.set_all_interface(db_get_interfaces(id, node_and_interface_id))
			nnode.name = name
			nnode.reload
			
			$host[ip]=nnode

			if node_and_snmp_monitor_id
				stmt=$db.prepare("SELECT snmp_state, oid_ref_name, oper, threshold, oidsev FROM node_and_snmp_monitor, snmp_monitor WHERE node_id=? and node_and_snmp_monitor.id=? and snmp_monitor_id=snmp_monitor.id")
				stmt.execute(id, node_and_snmp_monitor_id) do |rs|
					rs.each do |state, oidref, oper, threshold, sev|
						$host[ip].add_snmp_monitoring(state.to_i(), oidref, oper.to_i(), threshold, sev.to_i(),"init")
					end
				end
			end
			
			if node_and_wmi_monitor_id
				stmt=$db.prepare("SELECT wmi_state, wmi_ref_name, oper, value, wmisev FROM node_and_wmi_monitor, wmi_monitor WHERE node_id=? and node_and_wmi_monitor.id=? and wmi_monitor_id=wmi_monitor.id")
				stmt.execute(id, node_and_wmi_monitor_id) do |rs|
					rs.each do |state, wmiref, oper, val, sev|
						$host[ip].add_wmi_monitoring(state.to_i(), wmiref, oper.to_i(), val, sev.to_i(), "init")
					end
				end
			end

			if node_and_jmx_monitor_id
				db_get_jmx_monitoring_ids(ip, id, node_and_jmx_monitor_id)
			end			

			if node_and_snmptrap_monitor_id
				stmt=$db.prepare("SELECT snmptrap_state, snmptrap_ref, snmptrap_sev FROM node_and_snmptrap_monitor, snmptrap_monitor WHERE node_id=? and node_and_snmptrap_monitor.id=? and snmptrap_monitor_id=snmptrap_monitor.id")
				stmt.execute(id, node_and_snmptrap_monitor_id) do |rs|
					rs.each do |state, ref, sev|
						$host[ip].add_snmptrap_monitoring(state.to_i(), ref, sev.to_i(),"init")
					end
				end
			end
			
			if node_and_syslog_monitor_id
				stmt=$db.prepare("SELECT syslog_state, syslog_ref, syslog_sev FROM node_and_syslog_monitor, syslog_monitor WHERE node_id=? and node_and_syslog_monitor.id=? and syslog_monitor_id=syslog_monitor.id")
				stmt.execute(id, node_and_syslog_monitor_id) do |rs|
					rs.each do |state, ref, sev|
						$host[ip].add_syslog_monitoring(state.to_i(), ref, sev.to_i(),"init")
					end
				end
			end
			
      if node_and_custom_monitor_id
				stmt=$db.prepare("SELECT custom_state, custom_name, custom_param, custom_sev FROM node_and_custom_monitor, custom_monitor WHERE node_id=? and node_and_custom_monitor.id=? and custom_monitor_id=custom_monitor.id")
				stmt.execute(id, node_and_custom_monitor_id) do |rs|
					rs.each do |state, name, param, sev|
						$host[ip].add_custom_monitoring(state.to_i(), name, param, sev.to_i(), "init")
					end
				end
			end      
      
			$network[map_ip].add_host(nnode)
			$network[map_ip].update_status()			

			end
	end
end

#
#	read db schema version
#
def db_check_version_schema()
	stmt=$db.prepare("SELECT version FROM db_schema")
        stmt.execute() do |rs|
	    rs.each do |vers|
	       #converting SQLite3::ResultSet::ArrayWithTypesAndFields to float
	       if vers.join("\t").to_s.to_f != DB_SCHEMA_VERSION
	       	      $log.fatal("Wrong db version #{vers} expected #{DB_SCHEMA_VERSION}, backup your .db file and try again")
	              exit()
	       end
	    end
	end
        return 0	
end

#
#	read contacts information and fill hash
#
def db_read_contact()
	stmt=$db.prepare("SELECT firstname, lastname, email, phone, mobile, pager, im, title, organization, location, note FROM contact")
 	stmt.execute() do |rs|
		rs.each do |firstname, lastname, email, phone, mobile, pager, im, title, organization, location, note|
			$contact["#{firstname}|#{lastname}"]=Contact.new(firstname, lastname, email, phone, mobile, pager, im, title, organization, location, note)
		end
	end
	return 0
end

#
#	return contact id
#
def db_get_contact_id(firstname, lastname)
	stmt=$db.prepare("SELECT id FROM contact WHERE firstname=? and lastname=?")
	stmt.execute(firstname, lastname) do |rs|
		rs.each do |cid|
			if cid
				return cid
			end
		end
	end	
end

#
#	return array of firstname and lastname for a given contact
#
def db_get_contact_names(id)
	stmt=$db.prepare("SELECT firstname, lastname FROM contact WHERE id=?")
	stmt.execute(id) do |rs|
		rs.each do |firstname, lastname|
				return [firstname, lastname]
		end
	end	
end

#
#	write contacts information
#
def db_write_contact(firstname, lastname, email, phone, mobile, pager, im, title, organization, location, note)
	cid=db_get_contact_id(firstname, lastname)
	if cid
		stmt2=$db.prepare("UPDATE contact SET firstname=?, lastname=?, email=?, phone=?, mobile=?, pager=?, im=?, title=?, organization=?, location=?, note=? WHERE id=?")
		$db.transaction
		stmt2.execute(firstname, lastname, email, phone, mobile, pager, im, title, organization, location, note, cid)
		$db.commit
	else
		stmt2=$db.prepare("INSERT INTO contact VALUES ((SELECT IFNULL(MAX(id),0) FROM contact)+1, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
		$db.transaction
		stmt2.execute(firstname, lastname, email, phone, mobile, pager, im, title, organization, location, note)
		$db.commit
	end
end

#
#	del contacts information 
#	we don't need to check for node_and_contact value entries, they should have already been handled
#	so here we remove not USED contact
#
def db_del_contact(firstname, lastname)
	stmt=$db.prepare("DELETE from contact WHERE firstname=? and lastname=?")
	$db.transaction
	stmt.execute(firstname, lastname)
	$db.commit
end

def db_read_refs()
   OIDRef::read_db_oidref()
   WMIRef::read_db_wmiref()
   JMXRef::read_db_jmxref()
   SyslogRef::read_db_syslogref()
   SnmpTrapRef::read_db_snmptrapref()
end

#
# read and init component hash
#
def db_read(fic=SQLITE_DB)
	open_db(fic)
	db_check_version_schema()
	db_read_refs()
  db_set_custom_monitor()
	db_read_contact()	
	db_read_node()
	db_get_links()
end

#
#	create database tables
#
def create_db(fic=SQLITE_DB)
	$log.info("Create and fill DB")
	open_db(fic)
	begin
	sql = <<SQL
	

		CREATE TABLE service_monitor (
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			service_id INTEGER
				CONSTRAINT fk_service_id REFERENCES service(id),
			monitor INTEGER,
			service_sev INTEGER,
			PRIMARY KEY (id, node_id, service_id)
		);

		CREATE TABLE service (
			id INTEGER NOT NULL PRIMARY KEY,
			protocol VARCHAR(10),
			port INTEGER,
			name VARCHAR(50)
		);

		CREATE TABLE snmp_monitor (
			id INTEGER NOT NULL PRIMARY KEY,
			snmp_state BOOLEAN,
			oid_ref_name VARCHAR(50)
				CONSTRAINT fk_oid_ref_name REFERENCES oid_ref(name),				
			oper INTEGER,
			threshold INTEGER,
			oidsev INTEGER
		);

		CREATE TABLE node_and_snmp_monitor(
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			snmp_monitor_id INTEGER
				CONSTRAINT fk_snmp_monitor_id REFERENCES snmp_monitor(id),
			PRIMARY KEY (id, node_id, snmp_monitor_id)
		);

		CREATE TABLE wmi_monitor (
			id INTEGER NOT NULL PRIMARY KEY,
			wmi_state BOOLEAN,
			wmi_ref_name VARCHAR(50)
				CONSTRAINT fk_wmi_ref_name REFERENCES wmi_ref(name),				
			oper INTEGER,
			value INTEGER,
			wmisev INTEGER
		);

		CREATE TABLE jmx_monitor (
			id INTEGER NOT NULL PRIMARY KEY,
			jmx_state BOOLEAN,
			jmx_ref_name VARCHAR(50)
				CONSTRAINT fk_jmx_ref_name REFERENCES jmx_ref(name),
			jmx_protocol VARCHAR(10),
			jmx_port INTEGER,
			jmx_auth VARCHAR(20)
		);

		CREATE TABLE jmx_attr_monitor (
			id INTEGER NOT NULL PRIMARY KEY,
			name VARCHAR(50),
			oper INTEGER,
			value INTEGER,
			jmxsev INTEGER
		);

		CREATE TABLE node_and_wmi_monitor(
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			wmi_monitor_id INTEGER
				CONSTRAINT fk_wmi_monitor_id REFERENCES wmi_monitor(id),
			PRIMARY KEY (id, node_id, wmi_monitor_id)
		);

		CREATE TABLE node_and_jmx_monitor(
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			jmx_monitor_id INTEGER
				CONSTRAINT fk_jmx_monitor_id REFERENCES jmx_monitor(id),
			jmx_attr_monitor_id INTEGER
				CONSTRAINT fk_jmx_attr_monitor_id REFERENCES jmx_attr_monitor(id),
			PRIMARY KEY (id, node_id, jmx_monitor_id, jmx_attr_monitor_id)
		);

		CREATE TABLE snmptrap_monitor (
			id INTEGER NOT NULL PRIMARY KEY,
			snmptrap_state BOOLEAN,
			snmptrap_ref VARCHAR(50)
				CONSTRAINT fk_snmptrap_ref_name REFERENCES snmptrap_ref(name),	
			snmptrap_sev INTEGER
		);

		CREATE TABLE node_and_snmptrap_monitor(
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			snmptrap_monitor_id INTEGER
				CONSTRAINT fk_snmptrap_id REFERENCES snmptrap(id),
			PRIMARY KEY (id, node_id, snmptrap_monitor_id)
		);

		CREATE TABLE syslog_monitor (
			id INTEGER NOT NULL PRIMARY KEY,
			syslog_state BOOLEAN,
			syslog_ref VARCHAR(50)
				CONSTRAINT fk_syslog_ref_name REFERENCES syslog_ref(name),				
			syslog_sev INTEGER
		);

		CREATE TABLE node_and_syslog_monitor(
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			syslog_monitor_id INTEGER
				CONSTRAINT fk_syslog_monitor_id REFERENCES syslog_monitor(id),
			PRIMARY KEY (id, node_id, syslog_monitor_id)
		);

		CREATE TABLE custom_monitor(
			id INTEGER NOT NULL PRIMARY KEY,
      custom_name VARCHAR(50)      
		);
    
    CREATE TABLE node_and_custom_monitor(
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			custom_monitor_id INTEGER
				CONSTRAINT fk_custom_monitor_id REFERENCES custom_monitor(id),
      custom_state BOOLEAN,
      custom_param VARCHAR(512),
      custom_sev INTEGER,      
			PRIMARY KEY (id, node_id, custom_monitor_id)
		);    
    
		CREATE TABLE account (
			id INTEGER PRIMARY KEY,
			idname VARCHAR(50),
			login VARCHAR(50),
			password VARCHAR(50)
		);

		CREATE TABLE node_and_account (
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			account_id INTEGER
				CONSTRAINT fk_account_id REFERENCES account(id),
			PRIMARY KEY (id, node_id, account_id)
		);

		CREATE TABLE alert_override (
			id INTEGER PRIMARY KEY,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			alert_override_detail_id INTEGER
				CONSTRAINT fk_alert_override_detail_id REFERENCES alert_override_detail(id),
			alert_type_by_level VARCHAR(20)
		);
		
		CREATE TABLE alert_override_contact (
			id INTEGER PRIMARY KEY,
			contact_id INTEGER
				CONSTRAINT fk_contact REFERENCES contact(id)
		);
		
		CREATE TABLE alert_override_detail (
			id INTEGER PRIMARY KEY,
			mail_state BOOLEAN,
			mail_by_level_id INTEGER
				CONSTRAINT fk_alert_override_contact1 REFERENCES alert_override_contact(id),
			popup_state BOOLEAN,
			beep_state BOOLEAN,
			sound_file_path_by_level VARCHAR(255),
			syslog_state BOOLEAN,
			sms_state BOOLEAN,
			sms_num_by_level_id INTEGER
				CONSTRAINT fk_alert_override_contact2 REFERENCES alert_override_contact(id),
			im_state BOOLEAN,
			im_dest_by_level_id INTEGER
				CONSTRAINT fk_alert_override_contact3 REFERENCES alert_override_contact(id),
			script_state BOOLEAN,
			script_name_by_level VARCHAR(255),
			irc_state BOOLEAN
		);

		CREATE TABLE interface (
			id INTEGER NOT NULL PRIMARY KEY,
			name VARCHAR(10),
			link_status INTEGER,
			speed VARCHAR(50),
			type VARCHAR(50)
		);

		CREATE TABLE node_and_interface (
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			interface_id INTEGER
				CONSTRAINT fk_interface_id REFERENCES interface(id),
			PRIMARY KEY (id, node_id, interface_id)
		);

		CREATE TABLE node (    
			id INTEGER NOT NULL PRIMARY KEY,
                        uuid VARCHAR(36),
			nodetype VARCHAR(1),
			ip VARCHAR(15),
			netmask INTEGER,
			ipv6 VARCHAR(39),
			netmask6 INTEGER,
			mac VARCHAR(17),
			map INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			status INTEGER,
			avail_sev INTEGER,
			snmp_sev INTEGER,
			custom_sev INTEGER,
			trap_sev INTEGER,
                        service_sev INTEGER,
                        wmi_sev INTEGER,
                        syslog_sev INTEGER,
                        jmx_sev INTEGER,
                        service_monitor_id INTEGER
				CONSTRAINT fk_service_monitor_id REFERENCES service_monitor(id),
			node_and_snmp_monitor_id INTEGER
				CONSTRAINT fk_node_and_snmp_monitor_id REFERENCES node_and_snmp_monitor(id),
			node_and_wmi_monitor_id INTEGER
				CONSTRAINT fk_node_and_wmi_monitor_id REFERENCES node_and_wmi_monitor(id),
			node_and_jmx_monitor_id INTEGER
				CONSTRAINT fk_node_and_jmx_monitor_id REFERENCES node_and_jmx_monitor(id),
			node_and_snmptrap_monitor_id INTEGER
				CONSTRAINT fk_node_and_snmptrap_monitor_id REFERENCES node_and_snmptrap_monitor(id),
			node_and_syslog_monitor_id INTEGER
				CONSTRAINT fk_node_and_syslog_monitor_id REFERENCES node_and_syslog_monitor(id),
      node_and_custom_monitor_id INTEGER
				CONSTRAINT fk_node_and_custom_monitor_id REFERENCES node_and_custom_monitor(id),			      
      node_and_account_id INTEGER CONSTRAINT fk_node_and_account_id REFERENCES node_and_account(id),
			node_and_group_id INTEGER CONSTRAINT fk_node_and_group_id REFERENCES node_and_group(id),
			node_and_interface_id INTEGER CONSTRAINT fk_node_and_interface_id REFERENCES node_and_interface(id),
			x INTEGER,
			y INTEGER,
			icon VARCHAR(50),
			description VARCHAR(255),
			snmp_version VARCHAR(50),
			snmp_port VARCHAR(50),
			snmp_timeout VARCHAR(50),
			snmp_retry VARCHAR(50),
			type VARCHAR(50),
			os VARCHAR(50),
			name VARCHAR(50),
			dns_name VARCHAR(50),
			dns_domain VARCHAR(50),
			netbios_name VARCHAR(50),
			netbios_domain VARCHAR(50),
			node_size INTEGER,
			map_bg_type VARCHAR(20),
			map_bg_data VARCHAR(255)
		);

		CREATE TABLE link (
			id INTEGER PRIMARY KEY,
			link_state INTEGER,
			name VARCHAR(255),
			speed VARCHAR(50),
			node1 INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),	
			node2 INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id)
		);

		CREATE TABLE ngroup (
			id INTEGER PRIMARY KEY,
			name VARCHAR(50)
		);

		CREATE TABLE node_and_group (		
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			group_id INTEGER
				CONSTRAINT fk_group_id REFERENCES ngroup(id),
			PRIMARY KEY (id, node_id, group_id)
		);

		CREATE TABLE node_and_contact (		
			id INTEGER NOT NULL,
			node_id INTEGER
				CONSTRAINT fk_node_id REFERENCES node(id),
			contact_id INTEGER
				CONSTRAINT fk_contact_id REFERENCES contact(id),
			PRIMARY KEY (id, node_id, contact_id)
		);

		CREATE TABLE contact (		
			id INTEGER NOT NULL,
			firstname VARCHAR(50),
			lastname VARCHAR(50),
			email VARCHAR(50),
			phone VARCHAR(20),
			mobile VARCHAR(20),
			pager VARCHAR(20),
			im VARCHAR(50),
			title VARCHAR(255),
			organization VARCHAR(255),
			location VARCHAR(255),
			note VARCHAR(255),
			PRIMARY KEY (id)
		);
		
		CREATE TABLE db_schema (
			version REAL
		);
		
		INSERT INTO db_schema VALUES (#{DB_SCHEMA_VERSION});
		
		CREATE TABLE oid_ref (
			name VARCHAR(50) PRIMARY KEY,
			description VARCHAR(255),
			value VARCHAR(255)
		);

		INSERT INTO oid_ref VALUES ('Cpu', 'UCD-SNMP-MIB::ssCpuUser.0+UCD-SNMP-MIB::ssCpuSystem.0', '1.3.6.1.4.1.2021.11.9.0+1.3.6.1.4.1.2021.11.10.0');

		CREATE TABLE wmi_ref (
			name VARCHAR(50) PRIMARY KEY,
			description VARCHAR(255),
			value VARCHAR(255)
		);

		INSERT INTO wmi_ref VALUES ('Services', 'List Win32 Started Services', 'select Name from Win32_Service where Started=true');
		INSERT INTO wmi_ref VALUES ('Processes', 'List Win32 Processes', 'select * from Win32_Process');
		INSERT INTO wmi_ref VALUES ('Accounts', 'List Win32 Accounts', 'select * from Win32_UserAccount');

		CREATE TABLE jmx_ref (
			name VARCHAR(50) PRIMARY KEY,
			description VARCHAR(255),
			mbean VARCHAR(255),
			attribute VARCHAR(255)
		);
		INSERT INTO jmx_ref VALUES ('Memory', 'Memory MBean', 'java.lang:type=Memory', 'heap_memory_usage non_heap_memory_usage');
		INSERT INTO jmx_ref VALUES ('OS', 'OS MBean', 'java.lang:type=OperatingSystem', 'name');
		INSERT INTO jmx_ref VALUES ('Thread', 'Thread MBean', 'java.lang:type=Threading', 'thread_count');

		CREATE TABLE syslog_ref (
			name VARCHAR(50) PRIMARY KEY,
			description VARCHAR(255),
			value VARCHAR(255)
		);
		INSERT INTO syslog_ref VALUES ('Catch all', 'Trigger on all events', '(.+)');

		CREATE TABLE snmptrap_ref (
			name VARCHAR(50) PRIMARY KEY,
			description VARCHAR(255),
			value VARCHAR(255)
		);
		INSERT INTO snmptrap_ref VALUES ('Cold Start', '', 'snmp.0.0');
		INSERT INTO snmptrap_ref VALUES ('Warn Start', '', 'snmp.1.0');
		INSERT INTO snmptrap_ref VALUES ('Link Down', '', 'snmp.2.0');
		INSERT INTO snmptrap_ref VALUES ('Link Up', '', 'snmp.3.0');
		INSERT INTO snmptrap_ref VALUES ('Authentication failure', '', 'snmp.4.0');

SQL

			$db.execute_batch(sql)
			File.chmod(0600, fic)
		rescue SQLite3::SQLException => msg
			$log.fatal("Error #{msg} #{msg.backtrace.join("\n")}, can't create DB #{fic}")
			File.delete(fic)
			exit(1)
		end
		db_read_refs()
    db_set_custom_monitor()
end

#
# return nb lign of the file db
#

def nb_lign_db(fic=DEFAULTHOSTDB)
	if FileTest.exist?(fic)
		fic = File.new(fic,'r')
		lign=fic.gets
		nblign=1
		while lign
			lign=fic.gets
			nblign+=1
		end
		fic.close
	end
	return nblign
end

  class Listener
    include StreamListener

	 def init_service_attr()
		@protocol=nil
		@port=nil
		@service_name=nil
		@monitor=nil
		@service_sev=nil
	 end

	 def init_snmp_attr()
		@snmp_state=nil
		@oid_ref=nil
		@snmp_oper=nil
		@snmp_threshold=nil
		@oid_sev=nil
	 end

	 def init_wmi_attr()
		@wmi_state=nil
		@wmi_ref=nil
		@wmi_oper=nil
		@wmi_value=nil
		@wmi_sev=nil
	 end

	 def init_snmptrap_attr()
		@snmptrap_state=nil
		@snmptrap_ref=nil
		@trapsev=nil
	 end

	 def init_syslog_attr()
		@syslog_state=nil
		@syslog_ref=nil
		@syslog_sev=nil
	 end

    def init_attr()
                @uuid=nil
		@ip=nil
		@ipv6=nil
		@mask=nil
		@mask6=nil
		@mac=nil
		@dns_name=nil
		@netbios_name=nil
		@status=1
		@avail_sev=UNMANAGED
		@snmp_sev=UNMANAGED
		@custom_sev=UNMANAGED
		@trap_sev=UNMANAGED
		@service_sev=UNMANAGED
		@wmi_sev=UNMANAGED
		@syslog_sev=UNMANAGED
		@jmx_sev=UNMANAGED
    @x=0
		@y=0
		@icon=nil
		@description=nil
		@snmp_version=nil
		@snmp_port=nil
		@snmp_timeout=nil
		@snmp_retry=nil
		@type=nil
		@os=nil
		@map=nil
		@name=nil
		@dns_domain=nil
		@netbios_domain=nil
		@node_size=nil
		@map_bg_type=nil
		@map_bg_data=nil
		@custom_monitor=nil
		@inherit_from_global=nil
    end
    def tag_start(name, attributes)
      #puts "Start #{name}"
		#test if tag is a new node
		if name == "Host" || name == "Network"
			init_attr()
		elsif name == "services"
			@services=Hash.new
		elsif name == "service"
			init_service_attr()
		elsif name == "snmp_monitor"
			@snmp_monitor=Hash.new
		elsif name == "wmi_monitor"
			@wmi_monitor=Hash.new
		elsif name == "snmptrap_monitor"
			@snmptrap_monitor=Hash.new
		elsif name == "snmptrap"
			init_snmptrap_attr()
		elsif name == "syslog_monitor"
			@syslog_monitor=Hash.new
		elsif name == "syslog"
			init_syslog_attr()
		end
      @tag=name
    end
	def text(text)
		if @tag == "uuid"
			@uuid=text
			return
		end
                if @tag == "ip"
			@ip=text
			return
		end
                if @tag == "ipv6"
			@ipv6=text
			return
		end
		if @tag == "mask"
			@mask=text
			return	
		end
		if @tag == "mask6"
			@mask6=text
			return	
		end
		if @tag == "mac"
			@mac=text
			return
		end
		if @tag == "dns_name"
			@dns_name=text
			return
		end
		if @tag == "netbios_name"
			@netbios_name=text
			return
		end
		if @tag == "protocol"
			@protocol=text
			return
		end
		if @tag == "port"
			@port=text
			return
		end
		if @tag == "monitor"
			@monitor=text
			return
		end
		if @tag == "service_sev"
			@service_sev=text
			return
		end
		if @tag == "status"
			@status=text
			return
		end
		if @tag == "avail_sev"
			@avail_sev=text
			return
		end
		if @tag == "snmp_sev"
			@snmp_sev=text
			return
		end
		if @tag == "custom_sev"
			@custom_sev=text
			return
		end
		if @tag == "trap_sev"
			@trap_sev=text
			return
		end
		if @tag == "service_sev"
			@service_sev=text
			return
		end
		if @tag == "wmi_sev"
			@wmi_sev=text
			return
		end
		if @tag == "syslog_sev"
			@syslog_sev=text
			return
		end
		if @tag == "jmx_sev"
			@jmx_sev=text
			return
		end
    if @tag == "x"
			@x=text
			return
		end
		if @tag == "y"
			@y=text
			return
		end
		if @tag == "icon"
			@icon=text
			return
		end
		if @tag == "description"
			@description=text
			return
		end
		if @tag == "snmp_version"
			@snmp_version=text
			return
		end
		if @tag == "snmp_port"
			@snmp_port=text
			return
		end
		if @tag == "snmp_timeout"
			@snmp_timeout=text
			return
		end
		if @tag == "snmp_retry"
			@snmp_retry=text
			return
		end
		if @tag == "type"
			@type=text
			return
		end
		if @tag == "os"
			@os=text
			return
		end
		if @tag == "map"
			@map=text
			return
		end
		if @tag == "name"
			@name=text
			return
		end
		if @tag == "dns_domain"
			@dns_domain=text
			return
		end
		if @tag == "netbios_domain"
			@netbios_domain=text
			return
		end
		if @tag == "node_size"
			@node_size=text
			return
		end
		if @tag == "map_bg_type"
			@map_bg_type=text
			return
		end
		if @tag == "map_bg_data"
			if @map_bg_type == "color"
				@map_bg_data=text.split
			else
				@map_bg_data=text
			end
			return
		end
		if @tag == "inherit_from_global"
			@inherit_from_global=text
			return
		end
		if @tag == "snmp_state"
			@snmp_state=text
			return
		end
		if @tag == "oid_ref"
			@oid_ref=text
			return
		end
		if @tag == "snmp_oper"
			@snmp_oper=text
			return
		end
		if @tag == "snmp_threshold"
			@snmp_threshold=text
			return
		end
		if @tag == "oid_sev"
			@oid_sev=text
			return
		end
		if @tag == "wmi_state"
			@wmi_state=text
			return
		end
		if @tag == "wmi_ref"
			@wmi_ref=text
			return
		end
		if @tag == "wmi_oper"
			@wmi_oper=text
			return
		end
		if @tag == "wmi_value"
			@wmi_value=text
			return
		end
		if @tag == "wmi_sev"
			@wmi_sev=text
			return
		end		
		if @tag == "snmptrap_state"
			@snmptrap_state=text
			return
		end
		if @tag == "snmptrap_ref"
			@snmptrap_ref=text
			return
		end
		if @tag == "trapsev"
			@trapsev=text
			return
		end
		if @tag == "custom_monitor"
			@custom_monitor=text
			return
		end
		if @tag == "syslog_state"
			@syslog_state=text
			return
		end
		if @tag == "syslog_ref"
			@syslog_ref=text
			return
		end
		if @tag == "syslog_sev"
			@syslog_sev=text
			return
		end
		if text.chomp != ""
			puts "Unknown xml tag #{@tag} value #{text.chomp}"
		end
	end
    def tag_end(name)
#      puts "End #{name}"

		if name == "Host"
			#puts @ip, @mask.to_i, @map, @mac, @dns_name, @netbios_name, @port, @status.to_i, @x.to_i, @y.to_i, @icon, @description, @type, @os
			nnode=Host.new(@ip, @mask.to_i, @ipv6, @mask6.to_i, @map, @mac, @dns_name, @netbios_name, @services, @status.to_i, @x.to_i, @y.to_i, @icon, @description, @type, @os, @uuid)
			if @name != Node::NONAME_PROPERTY
				nnod.name = @name
			end
			
			nnode.set_severities(@avail_sev.to_i, @snmp_sev.to_i, @custom_sev.to_i, @trap_sev.to_i, @service_sev.to_i, @wmi_sev.to_i, @syslog_sev.to_i, @jmx_sev.to_i)
			  nnode.snmp_version=@snmp_version
			  nnode.snmp_port=@snmp_port
			  nnode.snmp_timeout=@snmp_timeout
			  nnode.snmp_retry=@snmp_retry

			nnode.reload
			$host[@ip]=nnode
			if defined?(@snmp_monitor) && (@snmp_monitor != nil)
				@snmp_monitor.each do |mon,tab|
					oidref, state, oper, threshold, sev = mon, tab[0], tab[1], tab[2], tab[3]
					$host[@ip].add_snmp_monitoring(state.to_i(), oidref, oper.to_i(), threshold, sev.to_i())
				end
			end

			if defined?(@wmi_monitor) && (@wmi_monitor != nil)
				@wmi_monitor.each do |mon,tab|
					wmiref, state, oper, val, sev = mon, tab[0], tab[1], tab[2], tab[3]
					$host[@ip].add_wmi_monitoring(state.to_i(), wmiref, oper.to_i(), val, sev.to_i())
				end
			end

			if defined?(@snmptrap_monitor) && (@snmptrap_monitor != nil)
				@snmptrap_monitor.each do |mon,tab|
					snmptrap_ref, state, trapsev = mon, tab[0], tab[1]
					$host[@ip].add_snmptrap_monitoring(state.to_i(), snmptrap_ref, trapsev.to_i())
				end
			end
			if defined?(@custom_monitor) && (@custom_monitor != nil)
				@custom_monitor.each do |name,tab|
					state, param, sev = tab[0], tab[1], tab[2]
					$host[@ip].add_custom_monitoring(state.to_i(), name, param, sev.to_i())
				end
			end      
			if defined?(@syslog_monitor) && (@syslog_monitor != nil)
				@syslog_monitor.each do |ref,tab|
					state, sev = tab[0], tab[1]
					$host[@ip].add_syslog_monitoring(state.to_i(), ref, sev.to_i())
				end
			end
			#we update network host list
			#we assume here networks are already loaded
			$network[@map].add_host(nnode)
			$network[@map].update_status()
			return
		end
		if name == "Network"
			#puts @ip, @mask
			#we assume here first network loaded is root network ROOTMAPADDR
			if @ip==ROOTMAPADDR && $network != nil && $network[@ip]!=nil
				#we already have a root map network node
			else
			  nnode=Network.new(@ip, @mask.to_i, @ipv6, @mask6.to_i, @map, @services, @status.to_i, @x.to_i, @y.to_i, @icon, @description, @type, @os, @name, @dns_domain, @netbios_domain, @uuid)
			  nnode.snmp_version=@snmp_version
			  nnode.snmp_port=@snmp_port
			  nnode.snmp_timeout=@snmp_timeout
			  nnode.snmp_retry=@snmp_retry

			  if @node_size != nil && isInteger(@node_size)
				  nnode.node_size=@node_size.to_i()
			  end
			  if @map_bg_type != ""
				  nnode.map_bg_type=@map_bg_type
				  nnode.map_bg_data=@map_bg_data
			  end
			  nnode.set_inherit_conf(i2b(@inherit_from_global))
			  nnode.reload
			  $network[@ip]=nnode
			end
 		   return
		end
		if name == "services"
			init_service_attr()
		end
		if name == "service"
			@service_name="?" unless @service_name != nil
			@monitor=1 unless @monitor != nil
			@services["#{@protocol}/#{@port}"] = Service.new(@protocol, @port, @service_name, @monitor, @service_sev) unless @services == nil
		end
		#for each snmp_monitor under block
		if name == "snmp"
			if defined?(@oid_ref) && (@oid_ref != nil)
				@snmp_monitor["#{@oid_ref}"]=[@snmp_state, @snmp_oper, @snmp_threshold, @oid_sev]
			end
			init_snmp_attr()
		end
		#for each wmi_monitor under block
		if name == "wmi"
			if defined?(@wmi_ref) && (@wmi_ref != nil)
				@wmi_monitor["#{@wmi_ref}"]=[@wmi_state, @wmi_oper, @wmi_value, @wmi_sev]
			end
			init_wmi_attr()
		end
		
		#for each snmptrap_monitor under block
		if name == "snmptrap"
			if defined?(@snmptrap_ref) && (@snmptrap_ref != nil)
				@snmptrap_monitor["#{@snmptrap_ref}"]=[@snmptrap_state, @trapsev]
			end
			init_snmptrap_attr()
		end
		#for each syslog_monitor under block
		if name == "syslog"
			if defined?(@syslog_ref) && (@syslog_ref != nil)
				@syslog_monitor["#{@syslog_ref}"]=[@syslog_state, @syslog_sev]
			end
			init_syslog_attr()
		end
    end
  end

def create_root_network()
    $log.info("Creating root network")
    if !exist_network(ROOTMAPADDR)
       $network[ROOTMAPADDR]=Network.new(ROOTMAPADDR)
       $network[ROOTMAPADDR].name="Root"
    end
end

def read_xml(fic=DEFAULTHOSTDB)
  set_busy()
  #we do not check here if the xml file is valid
  $log.info("Loading nodes")
  listener = Listener.new
  begin
    file=File.new(fic)
  rescue
    $log.error("File #{fic} not found")
    create_root_network()
    return
  end
  parser = Parsers::StreamParser.new(file, listener)
  parser.parse()
  $log.info("Node database rebuilt")
  #here we assume we have loaded all networks
  #we update network host list where in fact host is a network in network
  $network.each_key {|ip|
     host=$network[ip]
     $network[host.map].add_network(host) unless host.map == nil
  }
  set_unbusy()
end

def network_to_xml(ip)
  doc = Document.new
  doc.add_element("Network")

  i=1
  doc.root.add_element("uuid")
  doc.root.elements[i].text = $network[ip].uuid
  i+=1
  doc.root.add_element("ip")
  doc.root.elements[i].text = $network[ip].ip
  i+=1
  doc.root.add_element("mask")
  doc.root.elements[i].text = $network[ip].netmask
  i+=1
  doc.root.add_element("map")
  doc.root.elements[i].text = $network[ip].map
  i+=1
  doc.root.add_element("services")
  services=doc.root.elements[i]
  $network[ip].service.each_value {|serv|
    service=Element.new("service")
    service.add_element("protocol")
    service.elements["protocol"].text = serv.protocol
    service.add_element("port")
    service.elements["port"].text = serv.port
    service.add_element("name")
    service.elements["name"].text = serv.name
    service.add_element("monitor")
    service.elements["monitor"].text = serv.monitor
    services.elements << service
  }
  i+=1

  doc.root.add_element("status")
  doc.root.elements[i].text = $network[ip].status
  i+=1

  doc.root.add_element("x")
  doc.root.elements[i].text = $network[ip].x
  i+=1

  doc.root.add_element("y")
  doc.root.elements[i].text = $network[ip].y
  i+=1

  doc.root.add_element("icon")
  doc.root.elements[i].text = $network[ip].icon
  i+=1

  doc.root.add_element("description")
  doc.root.elements[i].text = $network[ip].description
  i+=1

  doc.root.add_element("snmp_version")
  doc.root.elements[i].text = $network[ip].snmp_version
  i+=1

  doc.root.add_element("snmp_port")
  doc.root.elements[i].text = $network[ip].snmp_port
  i+=1

  doc.root.add_element("snmp_port")
  doc.root.elements[i].text = $network[ip].snmp_port
  i+=1

  doc.root.add_element("snmp_retry")
  doc.root.elements[i].text = $network[ip].snmp_retry
  i+=1

  doc.root.add_element("type")
  doc.root.elements[i].text = $network[ip].type
  i+=1

  doc.root.add_element("os")
  doc.root.elements[i].text = $network[ip].os
  i+=1

  doc.root.add_element("name")
  doc.root.elements[i].text = $network[ip].name
  i+=1

  doc.root.add_element("dns_domain")
  doc.root.elements[i].text = $network[ip].dns_domain
  i+=1

  doc.root.add_element("netbios_domain")
  doc.root.elements[i].text = $network[ip].netbios_domain
  i+=1

  doc.root.add_element("node_size")
  doc.root.elements[i].text = $network[ip].node_size
  i+=1

  doc.root.add_element("map_bg_type")
  doc.root.elements[i].text = $network[ip].map_bg_type
  i+=1

  doc.root.add_element("map_bg_data")
  if $network[ip].map_bg_type == "color"
    doc.root.elements[i].text = $network[ip].map_bg_data.join(" ")
  else
    doc.root.elements[i].text = $network[ip].map_bg_data
  end
  i+=1

  doc.root.add_element("inherit_from_global")
  doc.root.elements[i].text = b2i($network[ip].inherit_from_global)
  i+=1

  return doc
end

def write_xml(fic_name="#{CONF_DIR}/#{DEFAULTHOSTDBFILE}")
  set_busy()
  $log.info("Saving nodes to XML file #{fic_name}")
  begin
    tf=Tempfile.new(File.basename(fic_name))
  rescue Exception=>msg
    $log.error("Error in write_xml when tried to generate a temp file name")
    return
  end


  #we write first of all the root of all network = ROOTMAPADDR
  tf.puts(network_to_xml(ROOTMAPADDR))

  #1st: we write the node network
  $network.each_key {|ip|
    if ip != ROOTMAPADDR
      tf.puts(network_to_xml(ip))
    end
  }

  #2nd: we write the node host
  $host.each_key {|ip|
    doc = Document.new
    doc.add_element("Host")

    i=1
    doc.root.add_element("uuid")
    doc.root.elements[i].text = $host[ip].uuid
    i+=1
    doc.root.add_element("ip")
    doc.root.elements[i].text = $host[ip].ip
    i+=1
    doc.root.add_element("mask")
    doc.root.elements[i].text = $host[ip].netmask
    i+=1
    doc.root.add_element("map")
    doc.root.elements[i].text = $host[ip].map
    i+=1
    doc.root.add_element("mac")
    doc.root.elements[i].text = $host[ip].mac
    i+=1
    doc.root.add_element("dns_name")
    doc.root.elements[i].text = $host[ip].dns_name
    i+=1
    doc.root.add_element("netbios_name")
    doc.root.elements[i].text = $host[ip].netbios_name
    i+=1
    doc.root.add_element("services")
    services=doc.root.elements[i]
    $host[ip].service.each_value {|serv|
      service=Element.new("service")
      service.add_element("protocol")
      service.elements["protocol"].text = serv.protocol
      service.add_element("port")
      service.elements["port"].text = serv.port
      service.add_element("name")
      service.elements["name"].text = serv.name
      service.add_element("monitor")
      service.elements["monitor"].text = serv.monitor
      services.elements << service
    }
    i+=1
    doc.root.add_element("avail_sev")
    doc.root.elements[i].text = $host[ip].avail_sev
    i+=1
    doc.root.add_element("snmp_sev")
    doc.root.elements[i].text = $host[ip].snmp_sev
    i+=1
    doc.root.add_element("custom_sev")
    doc.root.elements[i].text = $host[ip].custom_sev
    i+=1
    doc.root.add_element("trap_sev")
    doc.root.elements[i].text = $host[ip].trap_sev
    i+=1
    doc.root.add_element("service_sev")
    doc.root.elements[i].text = $host[ip].service_sev
    i+=1
    doc.root.add_element("wmi_sev")
    doc.root.elements[i].text = $host[ip].wmi_sev
    i+=1
    doc.root.add_element("syslog_sev")
    doc.root.elements[i].text = $host[ip].syslog_sev
    i+=1
    doc.root.add_element("jmx_sev")
    doc.root.elements[i].text = $host[ip].jmx_sev
    i+=1
    doc.root.add_element("x")
    doc.root.elements[i].text = $host[ip].x
    i+=1
    doc.root.add_element("y")
    doc.root.elements[i].text = $host[ip].y
    i+=1
    doc.root.add_element("icon")
    doc.root.elements[i].text = $host[ip].icon
    i+=1
    doc.root.add_element("description")
    doc.root.elements[i].text = $host[ip].description
    i+=1
    doc.root.add_element("snmp_version")
    doc.root.elements[i].text = $host[ip].snmp_version
    i+=1
    doc.root.add_element("snmp_port")
    doc.root.elements[i].text = $host[ip].snmp_port
    i+=1
    doc.root.add_element("snmp_timeout")
    doc.root.elements[i].text = $host[ip].snmp_timeout
    i+=1
    doc.root.add_element("snmp_retry")
    doc.root.elements[i].text = $host[ip].snmp_retry
    i+=1
    doc.root.add_element("type")
    doc.root.elements[i].text = $host[ip].type
    i+=1
    doc.root.add_element("os")
    doc.root.elements[i].text = $host[ip].os
    i+=1

    doc.root.add_element("snmp_monitor")
    snmp_monitor=doc.root.elements[i]
    $host[ip].snmp_monitoring.each_value {|mon|
      snmp=Element.new("snmp")
      snmp.add_element("snmp_state")
      snmp.elements["snmp_state"].text = mon.get_state()
      snmp.add_element("oid_ref")
      snmp.elements["oid_ref"].text = mon.oidref
      snmp.add_element("snmp_oper")
      snmp.elements["snmp_oper"].text = SnmpMonitoring::TESTOP.index(mon.oper)
      snmp.add_element("snmp_threshold")
      snmp.elements["snmp_threshold"].text = mon.threshold.to_s()
      snmp.add_element("oid_sev")
      snmp.elements["oid_sev"].text = mon.raw_severity
      snmp_monitor.elements << snmp
    }
    i+=1

    doc.root.add_element("wmi_monitor")
    wmi_monitor=doc.root.elements[i]
    $host[ip].wmi_monitoring.each_value {|mon|
      wmi=Element.new("wmi")
      wmi.add_element("wmi_state")
      wmi.elements["wmi_state"].text = mon.get_state()
      wmi.add_element("wmi_ref")
      wmi.elements["wmi_ref"].text = mon.wmiref
      wmi.add_element("wmi_oper")
      wmi.elements["wmi_oper"].text = WmiMonitoring::TESTOP.index(mon.oper)
      wmi.add_element("wmi_value")
      wmi.elements["wmi_value"].text = mon.value.to_s()
      wmi.add_element("wmi_sev")
      wmi.elements["wmi_sev"].text = mon.raw_severity
      wmi_monitor.elements << wmi
    }
    i+=1

    doc.root.add_element("snmptrap_monitor")
    snmptrap_monitor=doc.root.elements[i]
    $host[ip].snmptrap_monitoring.each_value {|mon|
      snmptrap=Element.new("snmptrap")
      snmptrap.add_element("snmptrap_state")
      snmptrap.elements["snmptrap_state"].text = mon.get_state()
      snmptrap.add_element("snmptrap_ref")
      snmptrap.elements["snmptrap_ref"].text = mon.get_snmptrap_ref()
      snmptrap.add_element("trapsev")
      snmptrap.elements["trapsev"].text = mon.raw_severity
      snmptrap_monitor.elements << snmptrap
    }
    i+=1
  
    doc.root.add_element("custom_monitor")
    custom_monitor=doc.root.elements[i]
    $host[ip].custom_monitoring.each_value {|mon|
      custom=Element.new("custom")
      custom.add_element("custom_state")
      custom.elements["custom_state"].text = mon.get_state()
      custom.add_element("custom_name")
      custom.elements["custom_name"].text = mon.get_name()
      custom.add_element("custom_param")
      custom.elements["custom_param"].text = mon.get_param()
      custom.add_element("custom_sev")
      custom.elements["custom_sev"].text = mon.raw_severity
      custom.elements << custom
    }    
    i+=1

    doc.root.add_element("syslog_monitor")
    syslog_monitor=doc.root.elements[i]
    $host[ip].syslog_monitoring.each_value {|mon|
      syslog=Element.new("syslog")
      syslog.add_element("syslog_state")
      syslog.elements["syslog_state"].text = mon.get_state()
      syslog.add_element("syslog_ref")
      syslog.elements["syslog_ref"].text = mon.get_syslog_ref()
      syslog.add_element("syslog_sev")
      syslog.elements["syslog_sev"].text = mon.raw_severity
      syslog_monitor.elements << syslog
    }
    i+=1

    tf.puts(doc)
  }
  #all seems good
  tf.close
  FileUtils.move(tf.path, fic_name)
  set_unbusy()
end
