class Host < Node
  attr_accessor :mac, :name, :dns_name, :dns_domain, :netbios_name, :netbios_domain, :node_view

  def initialize (ip="", mask=24, ipv6="", mask6=64, mp=nil, mac="(noarp)",n="(noname)", nn="(noname)",p=nil, s=UNKNOWN, xd=0 ,yd=0, i="", d=NODESCR_PROPERTY, t="host", o="unknown", uuid=create_uuid())

    super(ip, mask, ipv6, mask6, mp, p, s, xd, yd, i, d, t, o, uuid)
    @mac=mac
    @dns_name=n
    @netbios_name=nn
    @name=NONAME_PROPERTY
    @dns_domain=NONAME_PROPERTY
    @netbios_domain=NONAME_PROPERTY
    @node_view=nil

    if mp != nil
      if $network[mp].status == UNMANAGED
        self.status=UNMANAGED
      end
    end

    #create gui view
    create_node()
    #store node in db
    caller_func = caller[1][/`.*'/][1..-2]
    write_db() unless caller_func == "db_read_node"
  end

  def mac?()
    return mac != NOARP_PROPERTY
  end

  def hostname?()
      return name != NONAME_PROPERTY
  end

  def dns_name?()
    return dns_name != NONAME_PROPERTY
  end

  def netbios_name?()
    return netbios_name != NONAME_PROPERTY
  end

  def get_dns_name()
    if dns_name?() or @name == ""
       if $config.noshow_fqdn
	 dns_name_tab="#{@dns_name}".split(/\./)
	 return dns_name_tab[0]
       else
	 return @dns_name
       end
    else
       #if no fqdn set return the hostname
       return @name
    end
  end

  def refresh_dns_name()
    if $config.host_label_view == "dns_name"
      @node_view.set_text(get_dns_name())
    end
  end

  #
  # method for changing status (color) of the node
  # st is a $status_value
  #
  def change_status(st)
    if @status != st
      #status has changed
      @status=st
      @node_view.icon_color($status_color[$status_value.index(st)]) unless @node_view==nil
      @links.each {|l|
         l.update_status()
      }
      
      if $treenode_win.visible?
        $treenode_win.update_row(self)
      end
      #update status bar stats if not locked
      if !$event_win.updating_statusbar?()
         $event_win.update_statusbar()
      end
      #update status of the network unless st is UNMANAGED or UNKNOWN
      if (st != UNMANAGED) && (st != UNKNOWN)
        $network[@map].update_status()
      end
    end
  end

  def change_type(typ)
    if @type != typ
      @node_view.set_type(typ)
    end
    @type=typ
  end

  #
  # method for changing label of the node
  # lb is nil when we just want a node label refresh
  #
  def change_label(lb=nil)
    if $config.show_label
      if (@old_label != lb) or lb==nil
        @old_label=lb unless lb==nil
        lb=@old_label unless lb!=nil
        if lb == "mac_manufacturer" && mac?
          if  (@mac[0,1] >= '0' && @mac[0,1] <= '9')
            macplus=@mac.split(/:/)
            #0,1,2 are reserved for the manufacturer
            macbeg=""
            for i in 0..2
              if macplus[i].size == 1 #one digit
                macbeg="#{macbeg}0#{macplus[i]}"
              else
                macbeg="#{macbeg}#{macplus[i]}"
              end
            end
            manufacturer=find_manufacturer(macbeg)
            if manufacturer != nil
              macplus="#{manufacturer}:#{macplus[3]}:#{macplus[4]}:#{macplus[5]}"
              node_view.set_text("#{macplus.to_s}")
            end
          else
            node_view.set_text(get_dns_name())
          end
        else
          if ((lb == "mac")||(lb == "mac_manufacturer")) && !mac?
            node_view.set_text("#{@ip}")
          else
            if lb == "dns_name"
              node_view.set_text(get_dns_name())
            else
              node_view.set_text("#{eval lb}")
            end
          end
        end
      end
    end
  end

  def create_node()
    @old_label=""
    #set default label
    label=get_dns_name()
    if !dns_name? && ip != ""
      label="#{ip}"
    end

    if @map != nil
      #print "host ", $network[@map].ip, "\n"
      @node_view=NodeHostView.new($network[@map].map_view, self, $status_color[$status_value.index(@status)], "#{PIXMAP_PATH}/#{@icon}", label)
      #@node_view.hide()
      change_label($config.host_label_view)
      if !$config.show_label
        @node_view.hide_text()
      end
    end
  end #create_node

  def delete()
    #destroy the links
    del_all_links()
    #destroy the tooltips
    @node_view.hide_tooltips()
    #destroy the view
    @node_view.destroy()
    if $treenode_win.visible?
      $treenode_win.del_row(self)
    end
    #remove from db
    db_delete_node(HOSTNODE, ip, netmask)

    del_host(@ip)
  end

  #
  # 1 is avail 2 is snmp 3 is custom 4 is trap
  #
  def set_severities(avail, snmp, custom, trap, serv, wmi, syslog, jmx)
    @avail_sev=avail
    @snmp_sev=snmp
    @custom_sev=custom
    @trap_sev=trap
    @service_sev=serv
    @wmi_sev=wmi
    @syslog_sev=syslog
    @jmx_sev=jmx
    refresh_status()
  end

  #reload pixmap icon to apply changes
  def reload()
    if ($config.host_icon_view=="os")
      @icon="#{find_extension("os/#{@os}")}"
    else
      @icon="#{find_extension("type/#{@type}")}"
    end
    @node_view.set_image(@icon)
  end

  #
  # Return a table for the find tool
  #
  def to_t
    tmp=[]
    tmp.push(@ip,$status[$status_value.index(@status)], @mac, @dns_name, @netbios_name, @description, @type, @os)
    return tmp
  end

  def write_db()
    return db_insert_node_prp(uuid,HOSTNODE,ip,netmask,ipv6,netmask6,mac,map,status,avail_sev,snmp_sev,custom_sev,trap_sev, service_sev, wmi_sev, syslog_sev, jmx_sev, x,y,icon,description, nil, nil, nil,nil,type,os,"",dns_name,"",netbios_name,"",nil,nil,nil)
  end

end #class
