class Network < Node
  attr_accessor :name, :dns_domain, :netbios_domain, :host, :network, :node_view, :map_view, :node_size, :map_bg_type, :map_bg_data

  def initialize (ip="", mask=24, ipv6="", mask6=64, mp=nil, p=nil, s=UNKNOWN, xd=0 ,yd=0, i="", d=NODESCR_PROPERTY, t="network", o="unknown", n=NONAME_PROPERTY, dns_domain=NONAME_PROPERTY, netbios_domain=NONAME_PROPERTY, uuid=create_uuid())

    super(ip, mask, ipv6, mask6, mp, p, s, xd, yd, i, d, t, o, uuid)
    @name=n
    @dns_domain=NONAME_PROPERTY
    @netbios_domain=NONAME_PROPERTY
    @host=Hash.new
    @network=Hash.new
    @node_view=nil
    @node_size=nil
    @map_bg_type=nil
    @map_bg_data=nil

    if mp != nil and $network[mp]
      if $network[mp].status == UNMANAGED
        self.status=UNMANAGED
      end
    end
    #create gui view
    create_node()
    #store node in db
    #check from which the call is coming, if it's from db load fct we do nothing
    caller_func = caller[1][/`.*'/][1..-2]
    write_db() unless caller_func == "db_read_node"
  end

  #
  # h is a node
  #
  def add_host(h)
    if $treenode_win.visible?
      $treenode_win.add_row(h)
    end
    @host[h.ip]=h
  end

  def add_network(h)
    if $treenode_win.visible?
      $treenode_win.add_row(h)
    end
    @network[h.ip]=h
  end

  def del_host(hip)
    if $treenode_win.visible?
      $treenode_win.del_row(@host[hip])
    end
    @host.delete hip
  end

  def del_net(hip)
    if hip == @ip
      $log.error("Can't delete itself ip=#{ip}")
      return
    end
    if $treenode_win.visible?
      $treenode_win.del_row(@network[hip])
    end
    @network.delete hip
  end

  def contain_host(hip)
    @host.each_value {|node|
      if node.ip == hip
        return true
      end
    }
    return false
  end

  def contain_network(hip)
    #should be recursive
    @network.each_value {|node|
      if node.ip == hip
        return true
      else
        return node.contain_network(hip)
      end
    }
    return false
  end

  def get_node()
    return @host.merge(@network)
  end

  def nb_node()
    return nb_network() + nb_host()
  end

  def nb_host()
    return @host.size
  end

  def nb_network()
    return @network.size
  end


  def dns_domain?()
    return dns_domain != NONAME_PROPERTY
  end

  def netbios_domain?()
    return netbios_domain != NONAME_PROPERTY
  end

  def name?()
    return (@name != nil) && (@name != NONAME_PROPERTY) && (@name != "")
  end
  #
  # method for changing status (color) of the node
  #
  def change_status(st)
    if @status != st
      #status has changed
      @status=st
      @node_view.icon_color($status_color[$status_value.index(st)])
      @links.each {|l|
         l.update_status()
      }
      
      if $treenode_win.visible?
        $treenode_win.update_row(self)
      end

      #propagate status UNMANAGED  or UNKNOWN to all nodes includes in this network
      if (st == UNMANAGED) || (st == UNKNOWN)
        get_node().each_value {|node|
          node.change_status(st)
        }
      end
    end
  end

  #
  # update status depending on node host
  #
  def update_status()
    if @node_view != nil
      total_up=0
      total_other=0
      get_node().each_value{|n|
        if n.status == NORMAL
          total_up+=1
        elsif n.status != UNMANAGED
          total_other+=1
        end
      }
      total_host=total_up+total_other
      if total_host > 0
        percent_status_not_up=(total_other*100)/total_host
        if percent_status_not_up >= 50
          @status=CRITICAL
          @node_view.icon_color($status_color[$status_value.index(CRITICAL)])
        elsif percent_status_not_up == 0
          @status=NORMAL
          @node_view.icon_color($status_color[$status_value.index(NORMAL)])
        elsif	percent_status_not_up >= 25
          @status=MAJOR
          @node_view.icon_color($status_color[$status_value.index(MAJOR)])
        else
          @status=MINOR
          @node_view.icon_color($status_color[$status_value.index(MINOR)])
        end
        if $treenode_win.visible?
          $treenode_win.update_row(self)
        end
        #need to propagate to the network upstair
        $network[@map].update_status() unless @map == nil
      else
        @status=UNKNOWN
        @node_view.icon_color($status_color[$status_value.index(UNKNOWN)])
      end
    end
  end

  #
  # method for changing label of the node
  # lb is nil when we just want a node label refresh
  def change_label(lb=nil)
    if $config.show_label
      if (@old_label != lb) or lb==nil
        @old_label=lb unless lb==nil
        lb=@old_label unless lb!=nil
        if node_view
          node_view.set_text("#{eval lb}") unless ("#{eval lb}" == NODESCR_PROPERTY) || ("#{eval lb}" == NONAME_PROPERTY)
        end
      end
    end
  end

  def create_node()
    @old_label=""
    #set default label
    label="#{dns_domain}"
    if !dns_domain? && ip != ""
      label="#{ip}"
      if name?
        label="#{@name}"
      end
    end
    @map_view=Goo::CanvasGroup.new($canvas_root,{:x => 0.0,:y => 0.0})
    @map_view.visibility = Goo::CanvasItem::HIDDEN
    @node_view=nil
    if @map == nil
      #its the root network
      @node_view=NodeNetworkView.new($canvas_root, self, $status_color[$status_value.index(status)], "#{PIXMAP_PATH}/#{icon}", label)
    else
      #puts "create_node #{$network[@map]}"
      if $network[@map]
        @node_view=NodeNetworkView.new($network[@map].map_view, self, $status_color[$status_value.index(status)], "#{PIXMAP_PATH}/#{icon}", label)
      end
    end
    change_label($config.network_label_view)
    if @node_view
       if ($config.show_label &&((@map==nil)||(get_map()==@map)))
	 @node_view.show_text()
       else
	 @node_view.hide_text() unless !@node_view
       end
    end
  end #create_node

  def delete()
    #first need to destroy the links
    del_all_links()
    if @node_view
      #destroy the tooltips
      @node_view.hide_tooltips()
      #destroy the view
      @node_view.unselect()
      @node_view.destroy()
    end
    @node_view=nil

    #remove from db
    db_delete_node(NETWORKNODE, ip, netmask)

    del_network(@ip)
  end

  #reload pixmap icon to apply changes
  def reload()
    if ($config.network_icon_view=="os")
      @icon="#{find_extension("os/#{@os}")}"
    else
      @icon="#{find_extension("type/#{@type}")}"
    end
    @node_view.set_image(@icon) unless !@node_view
  end

  #
  # Return a table for the find tool
  #
  def to_t
    tmp=[]
    tmp.push(@ip,$status[$status_value.index(@status)], @name, @dns_domain, @netbios_domain, @description, @type, @os)
    return tmp
  end

  def write_db()
    return db_insert_node_prp(uuid,NETWORKNODE,ip,netmask,ipv6,netmask6,nil,map,status,avail_sev,snmp_sev,custom_sev,trap_sev, service_sev, wmi_sev, syslog_sev, jmx_sev, x,y,icon,description, snmp_version, snmp_port, snmp_timeout, snmp_retry,type,os,"",nil,dns_domain,nil,netbios_domain,nil,nil,nil)
  end

end #class
