require "#{GNMSLIB}"+'/notify/xmpp_bot'
require "#{GNMSLIB}"+'/notify/irc_bot'

#
# alarm module to warn admins
#

require 'net/smtp'

def alarm_handle(src_module, category, ip_str, summary, datetime)
  $log.debug("in alarm_handle")
  if $config.level != nil && ($LEVEL.index(category)>=$LEVEL.index($config.level))
    i=0
    level_alert_type=$level_alert_type
    if $host[ip_str] && (!$network[$host[ip_str].map].inherit_from_global)
      level_alert_type=$network[$host[ip_str].map].alert_type_by_level
    end
    level_alert_type[category].each do |alert|
      if alert == true
        case i
        when 0 then #MAIL
          alarm_mail("#{datetime} #{ip_str} #{summary}", ip_str, "#{src_module}", category)
        when 1 then #POPUP
          alarm_popup("#{src_module} #{datetime} #{ip_str} #{summary}")
        when 2 then #BEEP
          if $host[ip_str] && (!$network[$host[ip_str].map].inherit_from_global)
            if $network[$host[ip_str].map].sound_file_path_by_level[category] != nil
              if !$sound_is_played
                Thread.start { alarm_sound($network[$host[ip_str].map].sound_file_path_by_level[category]) }
              end
            else
              alarm_beep()
            end
          else
            if $config.sound_file_path_by_level[category] != nil
              if !$sound_is_played
                Thread.start { alarm_sound($config.sound_file_path_by_level[category]) }
              end
            else
              alarm_beep()
            end
          end
        when 3 then #SYSLOG
          text=sprintf("module='%s', severity='%s', time='%s', src='%s', descr='%s'", src_module, category, datetime, ip_str, summary)
          alarm_syslog(text)
        when 4 then #SMS / PAGER
          if ($config.sms_device != "") && ($config.sms_device != nil)
            if $host[ip_str] && (!$network[$host[ip_str].map].inherit_from_global) && ($network[$host[ip_str].map].sms_num_by_level[category] != "") && ($network[$host[ip_str].map].sms_num_by_level[category] != nil)
              alarm_sms($config.sms_device,$network[$host[ip_str].map].sms_num_by_level[category],"#{src_module} #{datetime} #{ip_str} #{summary}")
            elsif ($config.sms_num_by_level[category] != "") && ($config.sms_num_by_level[category] != nil)
              alarm_sms($config.sms_device,$config.sms_num_by_level[category],"#{src_module} #{datetime} #{ip_str} #{summary}")
            end
          end
        when 5 then #IM
          if $host[ip_str] && (!$network[$host[ip_str].map].inherit_from_global) && ($network[$host[ip_str].map].im_dest_by_level[category] != "") && ($network[$host[ip_str].map].im_dest_by_level[category] != nil)
            alarm_xmpp($network[$host[ip_str].map].im_dest_by_level[category],"#{src_module} #{datetime} #{ip_str} #{summary}")
          elsif ($config.im_dest_by_level[category] != "") && ($config.im_dest_by_level[category] != nil)
            alarm_xmpp($config.im_dest_by_level[category],"#{src_module} #{datetime} #{ip_str} #{summary}")
          end
        when 6 then #SCRIPT EXECUTION
          if $host[ip_str] && (!$network[$host[ip_str].map].inherit_from_global) && ($network[$host[ip_str].map].script_name_by_level[category] != "") && ($network[$host[ip_str].map].script_name_by_level[category] != nil)
            alarm_script($network[$host[ip_str].map].script_name_by_level[category], ip_str)
          elsif ($config.script_name_by_level[category] != "") && ($config.script_name_by_level[category] != nil)
            alarm_script($config.script_name_by_level[category], ip_str)
          else
            $log.error("Invalid script name for #{category}")
          end
        when 7 then #IRC
          alarm_irc("date=#{datetime} src=#{ip_str} descr=#{summary}")
        end
      end
      i+=1
    end
  end
end

def alarm_beep()
  printf "\a"
end

def alarm_sound(sound_file)
  if $sound_is_played
    return
  end

  if !FileTest.exist?("#{SOUND_DIR}/#{sound_file}")
    $log.error("Sound file not found at #{SOUND_DIR}/#{sound_file}")
    alarm_beep()
    return
  end

  if $defined_lib_gst
    $sound_is_played = true
      #create the pipeline
      pipeline = Gst::Pipeline.new
      # create a disk reader
      filesrc = Gst::ElementFactory.make("filesrc")
      filesrc.location = "#{SOUND_DIR}/#{sound_file}"

      decoder = Gst::ElementFactory.make("decodebin")

      if decoder == nil
	$log.error("gstreamer issue when initializing sound env")
	return
      end

      audio_convert = Gst::ElementFactory.make("audioconvert")
      audio_resample = Gst::ElementFactory.make("audioresample")
      audio_sink = Gst::ElementFactory.make("autoaudiosink")
      pipeline.add(filesrc, decoder, audio_convert, audio_resample, audio_sink)

      # link elements
      filesrc >> decoder
      audio_convert >> audio_resample >> audio_sink

      decoder.signal_connect("new-decoded-pad") do |element, pad|
	sink_pad = audio_convert["sink"]
	pad.link(sink_pad)
      end
            
      reading = true
      bus = pipeline.bus
      bus.add_watch do |bus, message|
	case message.type
	when Gst::Message::EOS
	  $sound_is_played = false
	  reading = false
	when Gst::Message::ERROR
	  #p message.parse
	  $sound_is_played = false
	  reading = false
	end
	true
      end

      # start playing
      pipeline.play
      while(reading) do 
        sleep(1)
      end
      pipeline.stop
      pipeline.set_state(Gst::State::NULL)
  else
    $log.error("Sound can't be played as sound lib gstreamer is not available")   
  end
end

def alarm_syslog(text)
  Syslog.log(Syslog::LOG_WARNING, text)
end

def alarm_popup(text)
  if $win.visible?
    dialog = Gtk::MessageDialog.new($win, Gtk::Dialog::MODAL,
    Gtk::MessageDialog::INFO,
    Gtk::MessageDialog::BUTTONS_OK,
    text)
    dialog.signal_connect('response') { dialog.destroy }
    dialog.show
  elsif $icon.embedded?
    #window is hidden and trayicon exists so we just blink the icon
    $icon.set_blinking(true)
  end
end

def alarm_mail(msg, ip, src_module=nil, sev=INFO)
  if src_module != "core"

    mail_server_addr=$config.mail_server_addr
    mail_server_port=$config.mail_server_port

    tmail_receipt=$config.mail_by_level[sev]
    if $host[ip] && (!$network[$host[ip].map].inherit_from_global)
      tmail_receipt=$network[$host[ip].map].mail_by_level[sev]
    end

    Thread.start {

      tmail_receipt.each{|contact|
        mail_dst=Contact::get_contact_email(contact.firstname, contact.lastname)
        if mail_dst != ""
          begin
            Net::SMTP.start( mail_server_addr, mail_server_port ) {|smtp|
              smtp.send_mail <<EndOfMail, "root@localhost", "#{mail_dst}"
              From: gnms admin <#{ENV["USER"]}@localhost>
              To: gnms contact <#{Contact::get_contact_email(contact.firstname, contact.lastname)}>
              Subject: gnms notification

              #{msg}
EndOfMail

            }
          rescue Exception => msg
            $log.error("Error #{sev} severity level #{msg} when sending mail to '#{mail_dst}' on '#{mail_server_addr}'/'#{mail_server_port}'")
          end

        else
          $log.error("Mail contact empty for #{contact.firstname} #{contact.lastname}")
        end
      }
    }

  end
end

def alarm_script(command, ip)
  if !FileTest.exist?("#{SCRIPT_DIR}/#{command}")
    $log.error("Invalid script path name #{SCRIPT_DIR}/#{command}")
  else
    `#{SCRIPT_DIR}/#{command} #{ip}`
  end
end

def alarm_sms(device, tnumber, txt)
  if $defined_lib_sms == false
    $log.error("Can't send SMS, notification type is disabled cause a library is missing")
  else
    Thread.start {
      tnumber.each{|contact|
        sms=Contact::get_contact_mobile(contact.firstname, contact.lastname)
        if sms != ""
          mySMS = SMSController.new(device, sms, txt)
          mySMS.sendSMS
        else
          $log.error("SMS contact empty for #{contact.firstname} #{contact.lastname}")
        end
      }
    }
  end
end

def alarm_xmpp(im_dest_addr, txt)
  if $im_daemon
    im_dest_addr.each{|contact|
      im=Contact::get_contact_im(contact.firstname, contact.lastname)
      if im != ""
        $im_daemon.add_in_queue(im, txt)
      else
        $log.error("IM contact empty for #{contact.firstname} #{contact.lastname}")
      end
    }
  end
end

def alarm_irc(txt)
  $irc_daemon.add_in_queue(txt) if $irc_daemon
end
