#!/usr/bin/env ruby
#doing my own class here as the xmpp project seems not maintain anymore

require 'socket'

class IrcBot

  def initialize()
    if $config.irc_daemon && ($config.irc_server != "")
      @buff = MsgBuffer.new(1)
      @connected = 0
      @channel = $config.irc_channel
      if !@channel or @channel == ""
        $log.error("IRC bot error: channel not set")
        return false
      end
      @server = $config.irc_server
      @port = isInteger($config.irc_port) ? $config.irc_port : 6667
      @botname = "gnmsbot"
      $log.info("Starting IRC bot #{@server}:#{@port} on channel ##{@channel}")
      begin
        @socket = TCPSocket.open(@server, @port)
      rescue Exception => msg
        $log.error("IRC bot error: #{msg}")
        return false
      end
      try_join(@botname)
      run()
    end
  end

  def try_join(nm)
    say "NICK #{nm}"
    say "USER #{nm} 0 * #{nm}"
    #try to join
    say "JOIN ##{@channel}"
  end
  
  def say(msg)
    $log.debug("IRC bot sent: #{msg}")
    @socket.puts msg
  end

  def say_to_chan(msg)
    $log.debug("IRC bot sent: #{msg}")
    say "PRIVMSG ##{@channel} :#{msg}"
  end

  def say_to_user(user, msg)
    $log.debug("IRC bot sent to #{user}: #{msg}")
    say "PRIVMSG #{user} :#{msg}"
  end
  
  def run
    @sender = Thread.start do
      while $config.irc_daemon
        send_messages if @buff.full?
        Thread.pass
        sleep(2)
      end
      $log.debug("end of IrcBot sender thread")
    end
    @receiver = Thread.start do
      until $config.irc_daemon and @socket.eof? do
        msg = @socket.gets
        if (msg)
          msg.chomp!
          $log.debug("IRC bot received: #{msg}")

          if msg.match(/^PING :(.*)$/)
            say "PONG #{$~[1]}"
            next
          end

          if msg.match(/:(.+)!~.+ PRIVMSG #{@botname} :(.*)$/)
            sender = $~[1]
            content = $~[2]
            if sender != ""
              if content and content.match(/^status$/)
                $host.each_value {|node|
                  say_to_user(sender, "IP #{node.ip} status is #{$status[$status_value.index(node.status)]}")
                }
              else
                say_to_user(sender, "status - display stats regarding known hosts")
              end
            end
            next
          end
          
          if msg.match(/PRIVMSG ##{@channel} :(.*)$/)
            content = $~[1]
            if content.match(/^beer$/)
              say_to_chan "Gimme a free beeeeeeeer"
            end
            next
          end
          
          if msg.match(/\s433\s/)
            if @connected == 0
              #nickname may be already in use
              @botname += "-#{rand(2**256).to_s(36)[0..4]}"
              $log.info("Irc bot renamed to #{@botname}")
              try_join(@botname)
              @connected = 1
            else
              $log.fatal("Irc bot can't connect #{msg}")
              stop()
            end
            next
          end
          
          if msg.match(/:End of \/MOTD/)
            #as there was a MOTD the join may have not succeed
            say "JOIN ##{@channel}"
            @connected = 1
            next
          end
        end
        Thread.pass
      end
      $log.debug("end of IrcBot receiver thread")
    end
  end

  def add_in_queue(msg)
    $log.debug("irc message added in queue")
    if  $config.irc_daemon
      @buff.put msg unless !@buff
    end
  end

  def stop()
    if defined?(@sender) && (@sender != nil)
      $log.info("Stopping IRC bot")
      quit()
      sleep(1)
      Thread.kill(@sender)
      Thread.kill(@receiver) if @receiver != nil
      @socket.close
      @connected = 0
      $log.debug("end of IrcBot threads")
    end
  end

  def quit
    say "PART ##{@channel} :Bye"
    say 'QUIT'
  end

  private
  def send_messages()
    while @connected and @buff.has_more?
      irc_msg=@buff.get
      if irc_msg != nil
        begin
          say_to_chan(irc_msg)
          sleep(1)
          Thread.pass if @buff.full?
        rescue Exception => msg
          #catch when stream is closed by close() method
          $log.error(msg)
        end
      end
    end
  end

end
