require "#{GNMSLIB}"+'/monitor/custom_plugin'

begin
  require 'net/ssh'
rescue LoadError
  $log.fatal("SshCpuLoadMonitor: you need to install net-ssh library for ruby")
  raise LoadError
end

class SshCpuLoadMonitor < CustomPlugin
  #declare static methods
  def SshCpuLoadMonitor::description()
    return "CPU load monitoring using Ssh connection to remotely read /proc/loadavg file"
  end
  def SshCpuLoadMonitor::name()
    return "CPU load monitoring using Ssh"
  end
  def SshCpuLoadMonitor::parameter()
    return 1.0
  end
  def SshCpuLoadMonitor::default_severity()
    return "MAJOR"
  end

  def monitor()
    host = get_node().ip
    cmd = "cat /proc/loadavg"
    max_cpuload = SshCpuLoadMonitor::parameter()
    service_name = "ssh"
    default_severity = SshCpuLoadMonitor::default_severity()

    if get_node().get_account_id(service_name)
      username, password = get_node().get_account_id(service_name)
      define_port = 22
      serv = get_node().service_by_name(service_name)
      if serv
         if serv.protocol == "tcp"
	         define_port = serv.port
	       else
	         raise("Could not run SshCpuLoadMonitor using other protocol than TCP")
	        end
      end
      
      param = get_node().custom_parameter(self.class.to_s)
      if (param != nil) and isDouble(param)
        max_cpuload = param.to_f
      end
      
      sev = get_node().custom_severity(self.class.to_s)
      if (sev != nil) and sev != ""
        default_severity = sev
      end
      
      begin
        Net::SSH.start(host, username, :password => password, :port => define_port) do |ssh|
          channel = ssh.open_channel do |ch|
            ch.exec cmd do |ch, success|
              raise("Could not run SshCpuLoadMonitor on #{host}") unless success
              ch.on_data do |c, data|
                #we remove the linefeed the end
                value=data.chomp.split(/\s/)[2]
            	if isDouble(value)
                  if value.to_f > max_cpuload.to_f
                    set_state(default_severity, "CPU load #{value} exceeded threshold #{max_cpuload}")
                  else
                    #set_state("NORMAL", "CPU load #{value.to_f * 100}%")
                    set_state("NORMAL")
                  end
                end
              end
            end
          end
          channel.wait
        end
      rescue Net::SSH::AuthenticationFailed
        raise("SSH authentication failed for user=#{username}")
      rescue Exception => msg
        raise(msg)
      end
    else
      $log.error("SSH login/password not set for ip=#{host}")
      set_state("UNKNOWN", "SSH login/password not set")
    end
  end
end
