require "#{GNMSLIB}"+'/monitor/custom_plugin'

begin
  require 'net/ssh'
rescue LoadError
  $log.fatal("SshDiskMonitor: you need to install net-ssh library for ruby")
  raise LoadError
end

class SshDiskMonitor < CustomPlugin
  #declare static methods
  def SshDiskMonitor::description()
    return "Disk usage monitoring using SSH connection to remotely launch df command"
  end
  def SshDiskMonitor::parameter()
    return
  end
  def SshDiskMonitor::name()
    return "Disk usage monitoring using SSH"
  end
  def SshDiskMonitor::default_severity()
    return "MAJOR"
  end

  def monitor()
    host=get_node().ip
    partition="/dev/sda1"
    hardlimit_value=90
    service_name = "ssh"

    if get_node().get_account_id(service_name)
      username, password = get_node().get_account_id(service_name)
      define_port = 22
      serv = get_node().service_by_name(service_name)
      if serv
         if serv.protocol == "tcp"
	    define_port = serv.port
	 else
	    raise("Could not run SshDiskMonitor using other protocol than TCP")
	 end
      end      
      begin
        Net::SSH.start(host, username, :password => password, :port => define_port) do |ssh|
          channel = ssh.open_channel do |ch|
            ch.exec "df -lhP|grep #{partition} |awk '{ print $5; }'" do |ch, success|
              raise("Could not run SshDiskMonitor on #{host}") unless success
              ch.on_data do |c, data|
                #we remove the linefeed and % char from the end
                value=data.chomp.chop
                if isInteger(value)
                  if value.to_i > hardlimit_value
                    set_state("CRITICAL", "Disk usage #{partition} exceeded > #{hardlimit_value}%")
                  else
                    set_state("NORMAL")
                  end
                end
              end
            end
          end
          channel.wait
        end
      rescue Net::SSH::AuthenticationFailed
        raise("SSH authentication failed for user=#{username}")    
      rescue Exception => msg
        raise(msg)
      end
    else
      $log.error("SSH login/password not set for ip=#{host}")      
      set_state("UNKNOWN", "SSH login/password not set")
    end
  end
end
