require "#{GNMSLIB}"+'/monitor/custom_plugin'
require 'date'

begin
  require 'net/ssh'
rescue LoadError
  $log.fatal("SshFgtAvExpriredMonitor: you need to install ssh library for ruby")
  raise LoadError
end

class SshFgtAvExpiredMonitor < CustomPlugin
  #declare static methods
  def SshFgtAvExpiredMonitor::description()
    return "Check Fortigate AV definition expiration date using SSH"
  end
  def SshFgtAvExpiredMonitor::parameter()
    return
  end
  def SshFgtAvExpiredMonitor::name()
    return "Fortigate AV definition"
  end
  def SshFgtAvExpiredMonitor::default_severity()
    return "MAJOR"
  end

  def monitor()

    host=get_node().ip
    nb_of_day=5
    cmd="get system fortiguard-service status"
    line_to_identify="Virus Definitions"
    service_name = "ssh"

    if get_node().get_account_id(service_name)    
      username, password = get_node().get_account_id(service_name)
      define_port = 22
      serv = get_node().service_by_name(service_name)
      if serv
         if serv.protocol == "tcp"
	    define_port = serv.port
	 else
	    raise("Could not run SshFgtAvExpiredMonitor using other protocol than TCP")
	 end
      end
      begin
        Net::SSH.start(host, username, :password => password, :port => define_port) do |ssh|
          channel = ssh.open_channel do |ch|
            ch.exec "#{cmd}" do |ch, success|
              raise("could not run SshFgtAvExpriredMonitor on #{host}") unless success
              ch.on_data do |c, data|
                #we remove the linefeed and % char from the end
                if data.match(line_to_identify)
                  #check last update
                  resultt=data.chomp.gsub(line_to_identify,"").split("\s")
                  last_update_day=resultt[1]
                  max_date=Date.today-nb_of_day
                  if Date.parse(last_update_day) < max_date
                    set_state("CRITICAL", "Fortigate AV definition expired, older than #{max_date}")
                  else
                    set_state("NORMAL")
                  end
                end
              end
            end
          end
          channel.wait
        end
      rescue Net::SSH::AuthenticationFailed
        raise("SSH authentication failed for user=#{username}")
      rescue Exception => msg
        raise(msg)
      end
    else
      $log.error("SSH login/password not set for ip=#{host}")
      set_state("UNKNOWN", "SSH login/password not set")
    end
  end
end
