require "#{GNMSLIB}"+'/monitor/custom_plugin'

begin
  require 'net/ssh'
rescue LoadError
  $log.fatal("SshPercentUsedRamMonitor: you need to install net-ssh library for ruby")
  raise LoadError
end

class SshPercentUsedRamMonitor < CustomPlugin
  #declare static methods
  def SshPercentUsedRamMonitor::description()
    return "% used RAM monitoring using Ssh connection to remotely run free cli cmd"
  end
  def SshPercentUsedRamMonitor::parameter()
    return
  end
  def SshPercentUsedRamMonitor::name()
    return "% used RAM monitoring using Ssh"
  end
  def SshPercentUsedRamMonitor::default_severity()
    return "MAJOR"
  end

  def monitor()
    host=get_node().ip
    cmd="/usr/bin/free | grep 'Mem'"
    max_mem_used=50
    service_name = "ssh"    

    if get_node().get_account_id(service_name)
      username, password = get_node().get_account_id(service_name)
      define_port = 22
      serv = get_node().service_by_name(service_name)
      if serv
         if serv.protocol == "tcp"
	    define_port = serv.port
	 else
	    raise("Could not run SshPercentUsedRamMonitor using other protocol than TCP")
	 end
      end      
      begin
        Net::SSH.start(host, username, :password => password, :port => define_port) do |ssh|
          channel = ssh.open_channel do |ch|
            ch.exec cmd do |ch, success|
              raise("Could not run SshPercentUsedRamMonitor on #{host}") unless success
              ch.on_data do |c, data|
                #we remove the linefeed the end
                value = data.chomp.split(/\s+/)
                mem_total = value[1]
                mem_used = value[2]
                
                result = mem_used.to_f * 100.0 / mem_total.to_f
                
            		if result >= 0.0 and result <= 100
                  if result > max_mem_used.to_f
                    set_state("CRITICAL", "Used RAM #{result.floor}% exceeded threshold #{max_mem_used}%")
                  else
                    set_state("NORMAL")
                end
                end
              end
            end
          end
          channel.wait
        end
      rescue Net::SSH::AuthenticationFailed
        raise("SSH authentication failed for user=#{username}")
      rescue Exception => msg
        raise(msg)
      end
    else
      $log.error("SSH login/password not set for ip=#{host}")
      set_state("UNKNOWN", "SSH login/password not set")
    end
  end
end
